@@ -3,8 +3,6 @@
 package flags
 
 import (
-	"time"
-
 	"github.com/spf13/viper"
 )
 
@@ -24,10 +22,6 @@ func LockTimeout() int {
 	return viper.GetInt("LOCK_TIMEOUT")
 }
 
-func BackfillBatchSize() int { return viper.GetInt("BACKFILL_BATCH_SIZE") }
-
-func BackfillBatchDelay() time.Duration { return viper.GetDuration("BACKFILL_BATCH_DELAY") }
-
 func SkipValidation() bool { return viper.GetBool("SKIP_VALIDATION") }
 
 func Role() string {
