@@ -39,11 +39,6 @@ type duplicatorStmtBuilder struct {
 const (
 	dataTypeMismatchErrorCode  pq.ErrorCode = "42804"
 	undefinedFunctionErrorCode pq.ErrorCode = "42883"
-
-	cCreateUniqueIndexSQL            = `CREATE UNIQUE INDEX CONCURRENTLY %s ON %s (%s)`
-	cSetDefaultSQL                   = `ALTER TABLE %s ALTER COLUMN %s SET DEFAULT %s`
-	cAlterTableAddCheckConstraintSQL = `ALTER TABLE %s ADD CONSTRAINT %s %s NOT VALID`
-	cAlterTableAddForeignKeySQL      = `ALTER TABLE %s ADD CONSTRAINT %s FOREIGN KEY (%s) REFERENCES %s (%s) ON DELETE %s`
 )
 
 // NewColumnDuplicator creates a new Duplicator for a column.
@@ -84,6 +79,12 @@ func (d *Duplicator) WithoutNotNull(columnName string) *Duplicator {
 	return d
 }
 
+// WithName sets the name of the new column.
+func (d *Duplicator) WithName(columnName, asName string) *Duplicator {
+	d.columns[columnName].asName = asName
+	return d
+}
+
 // Duplicate duplicates a column in the table, including all constraints and
 // comments.
 func (d *Duplicator) Duplicate(ctx context.Context) error {
@@ -128,12 +129,16 @@ func (d *Duplicator) Duplicate(ctx context.Context) error {
 		}
 	}
 
-	// Generate SQL to duplicate any unique constraints on the columns
-	// The constraint is duplicated by adding a unique index on the column concurrently.
+	// Create indexes for unique constraints on the columns concurrently.
 	// The index is converted into a unique constraint on migration completion.
-	for _, sql := range d.stmtBuilder.duplicateUniqueConstraints(d.withoutConstraint, colNames...) {
-		if _, err := d.conn.ExecContext(ctx, sql); err != nil {
-			return err
+	for _, uc := range d.stmtBuilder.table.UniqueConstraints {
+		if slices.Contains(d.withoutConstraint, uc.Name) {
+			continue
+		}
+		if duplicatedMember, constraintColumns := d.stmtBuilder.allConstraintColumns(uc.Columns, colNames...); duplicatedMember {
+			if err := createUniqueIndexConcurrently(ctx, d.conn, "", DuplicationName(uc.Name), d.stmtBuilder.table.Name, constraintColumns); err != nil {
+				return err
+			}
 		}
 	}
 
@@ -147,6 +152,13 @@ func (d *Duplicator) Duplicate(ctx context.Context) error {
 		}
 	}
 
+	// Generate SQL to duplicate any indexes on the columns.
+	for _, sql := range d.stmtBuilder.duplicateIndexes(d.withoutConstraint, colNames...) {
+		if _, err := d.conn.ExecContext(ctx, sql); err != nil {
+			return err
+		}
+	}
+
 	return nil
 }
 
@@ -157,28 +169,10 @@ func (d *duplicatorStmtBuilder) duplicateCheckConstraints(withoutConstraint []st
 			continue
 		}
 		if duplicatedConstraintColumns := d.duplicatedConstraintColumns(cc.Columns, colNames...); len(duplicatedConstraintColumns) > 0 {
-			stmts = append(stmts, fmt.Sprintf(cAlterTableAddCheckConstraintSQL,
-				pq.QuoteIdentifier(d.table.Name),
-				pq.QuoteIdentifier(DuplicationName(cc.Name)),
-				rewriteCheckExpression(cc.Definition, duplicatedConstraintColumns...),
-			))
-		}
-	}
-	return stmts
-}
-
-func (d *duplicatorStmtBuilder) duplicateUniqueConstraints(withoutConstraint []string, colNames ...string) []string {
-	stmts := make([]string, 0, len(d.table.UniqueConstraints))
-	for _, uc := range d.table.UniqueConstraints {
-		if slices.Contains(withoutConstraint, uc.Name) {
-			continue
-		}
-		if duplicatedMember, constraintColumns := d.allConstraintColumns(uc.Columns, colNames...); duplicatedMember {
-			stmts = append(stmts, fmt.Sprintf(cCreateUniqueIndexSQL,
-				pq.QuoteIdentifier(DuplicationName(uc.Name)),
-				pq.QuoteIdentifier(d.table.Name),
-				strings.Join(quoteColumnNames(constraintColumns), ", "),
-			))
+			sql := fmt.Sprintf("ALTER TABLE %s ADD ", pq.QuoteIdentifier(d.table.Name))
+			writer := ConstraintSQLWriter{Name: DuplicationName(cc.Name), SkipValidation: true}
+			sql += writer.WriteCheck(rewriteCheckExpression(cc.Definition, duplicatedConstraintColumns...), cc.NoInherit)
+			stmts = append(stmts, sql)
 		}
 	}
 	return stmts
@@ -191,14 +185,59 @@ func (d *duplicatorStmtBuilder) duplicateForeignKeyConstraints(withoutConstraint
 			continue
 		}
 		if duplicatedMember, constraintColumns := d.allConstraintColumns(fk.Columns, colNames...); duplicatedMember {
-			stmts = append(stmts, fmt.Sprintf(cAlterTableAddForeignKeySQL,
-				pq.QuoteIdentifier(d.table.Name),
-				pq.QuoteIdentifier(DuplicationName(fk.Name)),
-				strings.Join(quoteColumnNames(constraintColumns), ", "),
-				pq.QuoteIdentifier(fk.ReferencedTable),
-				strings.Join(quoteColumnNames(fk.ReferencedColumns), ", "),
-				fk.OnDelete,
-			))
+			sql := fmt.Sprintf("ALTER TABLE %s ADD ", pq.QuoteIdentifier(d.table.Name))
+			writer := ConstraintSQLWriter{
+				Name:    DuplicationName(fk.Name),
+				Columns: constraintColumns,
+			}
+			sql += writer.WriteForeignKey(
+				fk.ReferencedTable,
+				fk.ReferencedColumns,
+				ForeignKeyAction(fk.OnDelete),
+				ForeignKeyAction(fk.OnUpdate),
+				fk.OnDeleteSetColumns,
+				ForeignKeyMatchType(fk.MatchType),
+			)
+			stmts = append(stmts, sql)
+		}
+	}
+	return stmts
+}
+
+func (d *duplicatorStmtBuilder) duplicateIndexes(withoutConstraint []string, colNames ...string) []string {
+	stmts := make([]string, 0, len(d.table.Indexes))
+	for _, idx := range d.table.Indexes {
+		if slices.Contains(withoutConstraint, idx.Name) {
+			continue
+		}
+		if _, ok := d.table.UniqueConstraints[idx.Name]; ok && idx.Unique {
+			// unique constraints are duplicated as unique indexes
+			continue
+		}
+
+		if duplicatedMember, columns := d.allConstraintColumns(idx.Columns, colNames...); duplicatedMember {
+			stmtFmt := "CREATE INDEX CONCURRENTLY %s ON %s"
+			if idx.Unique {
+				stmtFmt = "CREATE UNIQUE INDEX CONCURRENTLY %s ON %s"
+			}
+			stmt := fmt.Sprintf(stmtFmt, pq.QuoteIdentifier(DuplicationName(idx.Name)), pq.QuoteIdentifier(d.table.Name))
+			if idx.Method != "" {
+				stmt += fmt.Sprintf(" USING %s", string(idx.Method))
+			}
+
+			stmt += fmt.Sprintf(" (%s)", strings.Join(quoteColumnNames(columns), ", "))
+
+			if storageParamStart := strings.Index(idx.Definition, " WITH ("); storageParamStart != -1 {
+				end := strings.Index(idx.Definition[storageParamStart:], ")")
+				stmt += idx.Definition[storageParamStart : storageParamStart+end+1]
+			}
+
+			if idx.Predicate != nil {
+				pred := strings.Replace(*idx.Predicate, strings.Join(idx.Columns, ", "), strings.Join(quoteColumnNames(columns), ", "), 1)
+				stmt += fmt.Sprintf(" WHERE %s", pred)
+			}
+
+			stmts = append(stmts, stmt)
 		}
 	}
 	return stmts
@@ -315,3 +354,36 @@ func errorIgnoringErrorCode(err error, code pq.ErrorCode) error {
 
 	return err
 }
+
+func alterSequenceOwnerToDuplicatedColumn(ctx context.Context, conn db.DB, tableName, columnName string) error {
+	sequenceName := getSequenceNameForColumn(ctx, conn, tableName, columnName)
+	if sequenceName == "" {
+		// No sequence for the column
+		return nil
+	}
+	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER SEQUENCE IF EXISTS %s OWNED BY %s.%s",
+		sequenceName,
+		pq.QuoteIdentifier(tableName),
+		pq.QuoteIdentifier(TemporaryName(columnName)),
+	))
+
+	return err
+}
+
+func getSequenceNameForColumn(ctx context.Context, conn db.DB, tableName, columnName string) string {
+	var sequenceName string
+	query := fmt.Sprintf(`
+		SELECT pg_get_serial_sequence('%s', '%s')
+	`, pq.QuoteIdentifier(tableName), columnName)
+	rows, err := conn.QueryContext(ctx, query)
+	if err != nil {
+		return ""
+	}
+	defer rows.Close()
+
+	if err := db.ScanFirstValue(rows, &sequenceName); err != nil {
+		return ""
+	}
+
+	return sequenceName
+}
