@@ -10,8 +10,8 @@ import (
 	"github.com/stretchr/testify/assert"
 
 	"github.com/xataio/pgroll/internal/testutils"
+	"github.com/xataio/pgroll/pkg/backfill"
 	"github.com/xataio/pgroll/pkg/migrations"
-	"github.com/xataio/pgroll/pkg/roll"
 )
 
 func TestAddColumn(t *testing.T) {
@@ -149,7 +149,7 @@ func TestAddColumn(t *testing.T) {
 					},
 				},
 			},
-			wantStartErr: migrations.BackfillNotPossibleError{Table: "users"},
+			wantStartErr: backfill.NotPossibleError{Table: "users"},
 		},
 		{
 			name: "add serial columns",
@@ -626,7 +626,7 @@ func TestAddForeignKeyColumn(t *testing.T) {
 									Name:     "fk_users_id",
 									Table:    "users",
 									Column:   "id",
-									OnDelete: migrations.ForeignKeyReferenceOnDeleteCASCADE,
+									OnDelete: migrations.ForeignKeyActionCASCADE,
 								},
 							},
 						},
@@ -635,7 +635,14 @@ func TestAddForeignKeyColumn(t *testing.T) {
 			},
 			afterStart: func(t *testing.T, db *sql.DB, schema string) {
 				// The foreign key constraint exists on the new table.
-				ValidatedForeignKeyMustExist(t, db, schema, "orders", "fk_users_id", withOnDeleteCascade())
+				ValidatedForeignKeyMustExistWithReferentialAction(
+					t,
+					db,
+					schema,
+					"orders",
+					"fk_users_id",
+					migrations.ForeignKeyActionCASCADE,
+					migrations.ForeignKeyActionNOACTION)
 
 				// Inserting a row into the referenced table succeeds.
 				MustInsert(t, db, schema, "01_create_table", "users", map[string]string{
@@ -668,7 +675,14 @@ func TestAddForeignKeyColumn(t *testing.T) {
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
 				// The foreign key constraint still exists on the new table
-				ValidatedForeignKeyMustExist(t, db, schema, "orders", "fk_users_id", withOnDeleteCascade())
+				ValidatedForeignKeyMustExistWithReferentialAction(
+					t,
+					db,
+					schema,
+					"orders",
+					"fk_users_id",
+					migrations.ForeignKeyActionCASCADE,
+					migrations.ForeignKeyActionNOACTION)
 
 				// Inserting a row into the referenced table succeeds.
 				MustInsert(t, db, schema, "02_add_column", "users", map[string]string{
@@ -1112,9 +1126,9 @@ func TestAddColumnWithUpSql(t *testing.T) {
 				// after rollback + restart + complete, all 'description' values are the backfilled ones.
 				res := MustSelect(t, db, schema, "02_add_column", "products")
 				assert.Equal(t, []map[string]any{
+					{"id": "c", "name": "cherries", "description": "CHERRIES"},
 					{"id": "a", "name": "apple", "description": "APPLE"},
 					{"id": "b", "name": "banana", "description": "BANANA"},
-					{"id": "c", "name": "cherries", "description": "CHERRIES"},
 				}, res)
 
 				// The trigger function has been dropped.
@@ -1200,6 +1214,491 @@ func TestAddNotNullColumnWithNoDefault(t *testing.T) {
 	})
 }
 
+func TestAddColumnWithVolatileAndNonVolatileDefaults(t *testing.T) {
+	t.Parallel()
+
+	ExecuteTests(t, TestCases{
+		{
+			name: "add nullable column with non-volatile default",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "users",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "text",
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_add_column",
+					Operations: migrations.Operations{
+						&migrations.OpAddColumn{
+							Table: "users",
+							Column: migrations.Column{
+								Name:     "age",
+								Type:     "integer",
+								Nullable: true,
+								Default:  ptr("0"),
+							},
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Inserting via both the old and the new views works
+				MustInsert(t, db, schema, "01_create_table", "users", map[string]string{
+					"id":   "1",
+					"name": "alice",
+				})
+				MustInsert(t, db, schema, "02_add_column", "users", map[string]string{
+					"id":   "2",
+					"name": "bob",
+					"age":  "21",
+				})
+
+				// Can insert a NULL value into the new view
+				MustInsert(t, db, schema, "02_add_column", "users", map[string]string{
+					"id":   "3",
+					"name": "carl",
+					"age":  "NULL",
+				})
+
+				// The old view has the expected rows
+				resOld := MustSelect(t, db, schema, "01_create_table", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "alice"},
+					{"id": 2, "name": "bob"},
+					{"id": 3, "name": "carl"},
+				}, resOld)
+
+				// The new view has the expected rows
+				resNew := MustSelect(t, db, schema, "02_add_column", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "alice", "age": 0},
+					{"id": 2, "name": "bob", "age": 21},
+					{"id": 3, "name": "carl", "age": nil},
+				}, resNew)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table has been cleaned up
+				TableMustBeCleanedUp(t, db, schema, "users", "age")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Inserting into the new view still works
+				MustInsert(t, db, schema, "02_add_column", "users", map[string]string{
+					"id":   "4",
+					"name": "dana",
+					"age":  "31",
+				})
+				MustInsert(t, db, schema, "02_add_column", "users", map[string]string{
+					"id":   "5",
+					"name": "earl",
+				})
+
+				// The new view has the expected rows
+				res := MustSelect(t, db, schema, "02_add_column", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "alice", "age": 0},
+					{"id": 2, "name": "bob", "age": 0},
+					{"id": 3, "name": "carl", "age": 0},
+					{"id": 4, "name": "dana", "age": 31},
+					{"id": 5, "name": "earl", "age": 0},
+				}, res)
+			},
+		},
+		{
+			name: "add not null column with non-volatile default",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "users",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "integer",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "text",
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_add_column",
+					Operations: migrations.Operations{
+						&migrations.OpAddColumn{
+							Table: "users",
+							Column: migrations.Column{
+								Name:     "age",
+								Type:     "integer",
+								Nullable: false,
+								Default:  ptr("0"),
+							},
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Inserting via both the old and the new views works
+				MustInsert(t, db, schema, "01_create_table", "users", map[string]string{
+					"id":   "1",
+					"name": "alice",
+				})
+				MustInsert(t, db, schema, "02_add_column", "users", map[string]string{
+					"id":   "2",
+					"name": "bob",
+					"age":  "21",
+				})
+
+				// Can't insert a NULL value into the new view
+				MustNotInsert(t, db, schema, "02_add_column", "users", map[string]string{
+					"id":   "3",
+					"name": "carl",
+					"age":  "NULL",
+				}, testutils.NotNullViolationErrorCode)
+
+				// The old view has the expected rows
+				resOld := MustSelect(t, db, schema, "01_create_table", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "alice"},
+					{"id": 2, "name": "bob"},
+				}, resOld)
+
+				// The new view has the expected rows
+				resNew := MustSelect(t, db, schema, "02_add_column", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "alice", "age": 0},
+					{"id": 2, "name": "bob", "age": 21},
+				}, resNew)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table has been cleaned up
+				TableMustBeCleanedUp(t, db, schema, "users", "age")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Inserting into the new view still works
+				MustInsert(t, db, schema, "02_add_column", "users", map[string]string{
+					"id":   "3",
+					"name": "dana",
+					"age":  "31",
+				})
+				MustInsert(t, db, schema, "02_add_column", "users", map[string]string{
+					"id":   "4",
+					"name": "earl",
+				})
+
+				// Can't insert a NULL value into the new view
+				MustNotInsert(t, db, schema, "02_add_column", "users", map[string]string{
+					"id":   "5",
+					"name": "frankie",
+					"age":  "NULL",
+				}, testutils.NotNullViolationErrorCode)
+
+				// The new view has the expected rows
+				res := MustSelect(t, db, schema, "02_add_column", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "alice", "age": 0},
+					{"id": 2, "name": "bob", "age": 0},
+					{"id": 3, "name": "dana", "age": 31},
+					{"id": 4, "name": "earl", "age": 0},
+				}, res)
+			},
+		},
+		{
+			name: "add nullable column with volatile default",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "users",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "text",
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_create_volatile_function",
+					Operations: migrations.Operations{
+						&migrations.OpRawSQL{
+							Up: `CREATE FUNCTION public.always_ten() RETURNS integer AS $$ BEGIN RETURN 10; END; $$ LANGUAGE plpgsql VOLATILE;`,
+						},
+					},
+				},
+				{
+					Name: "03_add_column",
+					Operations: migrations.Operations{
+						&migrations.OpAddColumn{
+							Table: "users",
+							Up:    "public.always_ten()",
+							Column: migrations.Column{
+								Name:     "age",
+								Type:     "integer",
+								Nullable: true,
+								Default:  ptr("public.always_ten()"),
+							},
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Inserting via both the old and the new views works
+				MustInsert(t, db, schema, "02_create_volatile_function", "users", map[string]string{
+					"id":   "1",
+					"name": "alice",
+				})
+				MustInsert(t, db, schema, "03_add_column", "users", map[string]string{
+					"id":   "2",
+					"name": "bob",
+					"age":  "21",
+				})
+
+				// Can insert a NULL value into the new view
+				MustInsert(t, db, schema, "03_add_column", "users", map[string]string{
+					"id":   "3",
+					"name": "carl",
+					"age":  "NULL",
+				})
+
+				// The old view has the expected rows
+				resOld := MustSelect(t, db, schema, "02_create_volatile_function", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "alice"},
+					{"id": 2, "name": "bob"},
+					{"id": 3, "name": "carl"},
+				}, resOld)
+
+				// The new view has the expected rows
+				resNew := MustSelect(t, db, schema, "03_add_column", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "alice", "age": 10},
+					{"id": 2, "name": "bob", "age": 21},
+					{"id": 3, "name": "carl", "age": nil},
+				}, resNew)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table has been cleaned up
+				TableMustBeCleanedUp(t, db, schema, "users", "age")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Inserting into the new view still works
+				MustInsert(t, db, schema, "03_add_column", "users", map[string]string{
+					"id":   "4",
+					"name": "dana",
+				})
+				MustInsert(t, db, schema, "03_add_column", "users", map[string]string{
+					"id":   "5",
+					"name": "earl",
+					"age":  "51",
+				})
+
+				// Can insert a NULL value into the new view
+				MustInsert(t, db, schema, "03_add_column", "users", map[string]string{
+					"id":   "6",
+					"name": "frankie",
+					"age":  "NULL",
+				})
+
+				// The new view has the expected rows
+				res := MustSelect(t, db, schema, "03_add_column", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "alice", "age": 10},
+					{"id": 2, "name": "bob", "age": 10},
+					{"id": 3, "name": "carl", "age": 10},
+					{"id": 4, "name": "dana", "age": 10},
+					{"id": 5, "name": "earl", "age": 51},
+					{"id": 6, "name": "frankie", "age": nil},
+				}, res)
+			},
+		},
+		{
+			name: "add not null column with volatile default",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "users",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "text",
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_create_volatile_function",
+					Operations: migrations.Operations{
+						&migrations.OpRawSQL{
+							Up: `CREATE FUNCTION public.always_ten() RETURNS integer AS $$ BEGIN RETURN 10; END; $$ LANGUAGE plpgsql VOLATILE;`,
+						},
+					},
+				},
+				{
+					Name: "03_add_column",
+					Operations: migrations.Operations{
+						&migrations.OpAddColumn{
+							Table: "users",
+							Up:    "public.always_ten()",
+							Column: migrations.Column{
+								Name:     "age",
+								Type:     "integer",
+								Nullable: false,
+								Default:  ptr("public.always_ten()"),
+							},
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Inserting via both the old and the new views works
+				MustInsert(t, db, schema, "02_create_volatile_function", "users", map[string]string{
+					"id":   "1",
+					"name": "alice",
+				})
+				MustInsert(t, db, schema, "03_add_column", "users", map[string]string{
+					"id":   "2",
+					"name": "bob",
+					"age":  "21",
+				})
+
+				// Can't insert a NULL value into the new view
+				MustNotInsert(t, db, schema, "03_add_column", "users", map[string]string{
+					"id":   "3",
+					"name": "carl",
+					"age":  "NULL",
+				}, testutils.CheckViolationErrorCode)
+
+				// The old view has the expected rows
+				resOld := MustSelect(t, db, schema, "02_create_volatile_function", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "alice"},
+					{"id": 2, "name": "bob"},
+				}, resOld)
+
+				// The new view has the expected rows
+				resNew := MustSelect(t, db, schema, "03_add_column", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "alice", "age": 10},
+					{"id": 2, "name": "bob", "age": 21},
+				}, resNew)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table has been cleaned up
+				TableMustBeCleanedUp(t, db, schema, "users", "age")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Inserting into the new view still works
+				MustInsert(t, db, schema, "03_add_column", "users", map[string]string{
+					"id":   "3",
+					"name": "carl",
+					"age":  "31",
+				})
+				MustInsert(t, db, schema, "03_add_column", "users", map[string]string{
+					"id":   "4",
+					"name": "dana",
+				})
+
+				// Can't insert a NULL value into the new view
+				MustNotInsert(t, db, schema, "03_add_column", "users", map[string]string{
+					"id":   "5",
+					"name": "earl",
+					"age":  "NULL",
+				}, testutils.NotNullViolationErrorCode)
+
+				// The new view has the expected rows
+				res := MustSelect(t, db, schema, "03_add_column", "users")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "alice", "age": 10},
+					{"id": 2, "name": "bob", "age": 10},
+					{"id": 3, "name": "carl", "age": 31},
+					{"id": 4, "name": "dana", "age": 10},
+				}, res)
+			},
+		},
+		{
+			name: "up expression must be the default expression when adding a column with a volatile default",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "users",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "text",
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_create_volatile_function",
+					Operations: migrations.Operations{
+						&migrations.OpRawSQL{
+							Up: `CREATE FUNCTION public.always_ten() RETURNS integer AS $$ BEGIN RETURN 10; END; $$ LANGUAGE plpgsql VOLATILE;`,
+						},
+					},
+				},
+				{
+					Name: "03_add_column",
+					Operations: migrations.Operations{
+						&migrations.OpAddColumn{
+							Table: "users",
+							Up:    "'incorrect - must be the same as the column default'",
+							Column: migrations.Column{
+								Name:     "age",
+								Type:     "integer",
+								Nullable: false,
+								Default:  ptr("public.always_ten()"),
+							},
+						},
+					},
+				},
+			},
+			wantStartErr: migrations.UpSQLMustBeColumnDefaultError{Column: "age"},
+		},
+	})
+}
+
 func TestAddColumnValidation(t *testing.T) {
 	t.Parallel()
 
@@ -1307,6 +1806,44 @@ func TestAddColumnValidation(t *testing.T) {
 			},
 			wantStartErr: migrations.ColumnAlreadyExistsError{Table: "users", Name: "name"},
 		},
+		{
+			name: "column must be valid (not missing its type)",
+			migrations: []migrations.Migration{
+				addTableMigration,
+				{
+					Name: "02_add_column",
+					Operations: migrations.Operations{
+						&migrations.OpAddColumn{
+							Table: "users",
+							Column: migrations.Column{
+								Name: "description",
+								// Missing type
+							},
+						},
+					},
+				},
+			},
+			wantStartErr: migrations.ColumnIsInvalidError{Table: "users", Name: "description"},
+		},
+		{
+			name: "column must be valid (not missing its name)",
+			migrations: []migrations.Migration{
+				addTableMigration,
+				{
+					Name: "02_add_column",
+					Operations: migrations.Operations{
+						&migrations.OpAddColumn{
+							Table: "users",
+							Column: migrations.Column{
+								// Missing name
+								Type: "text",
+							},
+						},
+					},
+				},
+			},
+			wantStartErr: migrations.ColumnIsInvalidError{Table: "users", Name: ""},
+		},
 		{
 			name: "up SQL is mandatory when adding a NOT NULL column with no DEFAULT",
 			migrations: []migrations.Migration{
@@ -1401,7 +1938,7 @@ func TestAddColumnValidation(t *testing.T) {
 					},
 				},
 			},
-			wantStartErr: migrations.BackfillNotPossibleError{Table: "users"},
+			wantStartErr: backfill.NotPossibleError{Table: "users"},
 		},
 		{
 			name: "table with a unique not null column can be backfilled",
@@ -1558,118 +2095,18 @@ func TestAddColumnWithComment(t *testing.T) {
 	}})
 }
 
-func TestAddColumnDefaultTransformation(t *testing.T) {
-	t.Parallel()
-
-	sqlTransformer := testutils.NewMockSQLTransformer(map[string]string{
-		"'default value 1'": "'rewritten'",
-		"'default value 2'": testutils.MockSQLTransformerError,
-	})
-
-	ExecuteTests(t, TestCases{
-		{
-			name: "column default is rewritten by the SQL transformer",
-			migrations: []migrations.Migration{
-				{
-					Name: "01_create_table",
-					Operations: migrations.Operations{
-						&migrations.OpCreateTable{
-							Name: "users",
-							Columns: []migrations.Column{
-								{
-									Name: "id",
-									Type: "serial",
-									Pk:   true,
-								},
-							},
-						},
-					},
-				},
-				{
-					Name: "02_add_column",
-					Operations: migrations.Operations{
-						&migrations.OpAddColumn{
-							Table: "users",
-							Column: migrations.Column{
-								Name:    "name",
-								Type:    "text",
-								Default: ptr("'default value 1'"),
-							},
-						},
-					},
-				},
-			},
-			afterStart: func(t *testing.T, db *sql.DB, schema string) {
-				// Insert some data into the table
-				MustInsert(t, db, schema, "02_add_column", "users", map[string]string{
-					"id": "1",
-				})
-
-				// Ensure the row has the rewritten default value.
-				rows := MustSelect(t, db, schema, "02_add_column", "users")
-				assert.Equal(t, []map[string]any{
-					{"id": 1, "name": "rewritten"},
-				}, rows)
-			},
-			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
-			},
-			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
-				// Ensure the row has the rewritten default value.
-				rows := MustSelect(t, db, schema, "02_add_column", "users")
-				assert.Equal(t, []map[string]any{
-					{"id": 1, "name": "rewritten"},
-				}, rows)
-			},
-		},
-		{
-			name: "operation fails when the SQL transformer returns an error",
-			migrations: []migrations.Migration{
-				{
-					Name: "01_create_table",
-					Operations: migrations.Operations{
-						&migrations.OpCreateTable{
-							Name: "users",
-							Columns: []migrations.Column{
-								{
-									Name: "id",
-									Type: "serial",
-									Pk:   true,
-								},
-							},
-						},
-					},
-				},
-				{
-					Name: "02_add_column",
-					Operations: migrations.Operations{
-						&migrations.OpAddColumn{
-							Table: "users",
-							Column: migrations.Column{
-								Name:    "name",
-								Type:    "text",
-								Default: ptr("'default value 2'"),
-							},
-						},
-					},
-				},
-			},
-			wantStartErr: testutils.ErrMockSQLTransformer,
-		},
-	}, roll.WithSQLTransformer(sqlTransformer))
-}
-
-func TestAddColumnToATableCreatedInTheSameMigration(t *testing.T) {
+func TestAddColumnInMultiOperationMigrations(t *testing.T) {
 	t.Parallel()
 
 	ExecuteTests(t, TestCases{
 		{
-			name: "add column to newly created table",
+			name: "create table, add column",
 			migrations: []migrations.Migration{
 				{
-					Name: "01_add_table",
+					Name: "01_multi_operation",
 					Operations: migrations.Operations{
 						&migrations.OpCreateTable{
-							Name: "users",
+							Name: "items",
 							Columns: []migrations.Column{
 								{
 									Name: "id",
@@ -1683,47 +2120,194 @@ func TestAddColumnToATableCreatedInTheSameMigration(t *testing.T) {
 							},
 						},
 						&migrations.OpAddColumn{
-							Table: "users",
+							Table: "items",
 							Column: migrations.Column{
-								Name:     "age",
-								Type:     "integer",
-								Nullable: false,
-								Default:  ptr("18"),
-								Check: &migrations.CheckConstraint{
-									Name:       "age_check",
-									Constraint: "age >= 18",
-								},
-								Comment: ptr("the age of the user"),
+								Name: "description",
+								Type: "text",
 							},
+							Up: "UPPER(name)",
 						},
 					},
 				},
 			},
 			afterStart: func(t *testing.T, db *sql.DB, schema string) {
-				// Inserting into the new column on the new table works.
-				MustInsert(t, db, schema, "01_add_table", "users", map[string]string{
-					"name": "Alice", "age": "30",
+				// Can insert into the view in the new schema (the only version schema)
+				MustInsert(t, db, schema, "01_multi_operation", "items", map[string]string{
+					"name":        "apples",
+					"description": "green",
 				})
 
-				// Inserting a value that doesn't meet the check constraint fails.
-				MustNotInsert(t, db, schema, "01_add_table", "users", map[string]string{
-					"name": "Bob", "age": "8",
-				}, testutils.CheckViolationErrorCode)
+				// The table has the expected rows
+				rows := MustSelect(t, db, schema, "01_multi_operation", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "description": "green"},
+				}, rows)
 			},
 			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table no longer exists
+				TableMustNotExist(t, db, schema, "items")
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
-				// Inserting into the new column on the new table works.
-				MustInsert(t, db, schema, "01_add_table", "users", map[string]string{
-					"name": "Bob", "age": "31",
+				// Can insert into the view in the new schema (the only version schema)
+				MustInsert(t, db, schema, "01_multi_operation", "items", map[string]string{
+					"name":        "bananas",
+					"description": "yellow",
 				})
 
-				// Inserting a value that doesn't meet the check constraint fails.
-				MustNotInsert(t, db, schema, "01_add_table", "users", map[string]string{
-					"name": "Carl", "age": "8",
-				}, testutils.CheckViolationErrorCode)
+				// The table has the expected rows
+				rows := MustSelect(t, db, schema, "01_multi_operation", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "bananas", "description": "yellow"},
+				}, rows)
 			},
 		},
+		{
+			name: "rename table, add column",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "items",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "serial",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "varchar(255)",
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_multi_operation",
+					Operations: migrations.Operations{
+						&migrations.OpRenameTable{
+							From: "items",
+							To:   "products",
+						},
+						&migrations.OpAddColumn{
+							Table: "products",
+							Column: migrations.Column{
+								Name: "description",
+								Type: "text",
+							},
+							Up: "UPPER(name)",
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert into the new table in the new schema using its new name
+				MustInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"name":        "apples",
+					"description": "green",
+				})
+
+				// Can't insert into the new table in the new schema using its old name
+				MustNotInsert(t, db, schema, "02_multi_operation", "items", map[string]string{
+					"name":        "bananas",
+					"description": "yellow",
+				}, testutils.UndefinedTableErrorCode)
+
+				// The table has the expected rows in the old schema
+				rows := MustSelect(t, db, schema, "01_create_table", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples"},
+				}, rows)
+
+				// The table has the expected rows in the new schema
+				rows = MustSelect(t, db, schema, "02_multi_operation", "products")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "description": "green"},
+				}, rows)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert into the old table in the old schema using its old name
+				MustInsert(t, db, schema, "01_create_table", "items", map[string]string{
+					"name": "bananas",
+				})
+
+				// The temporary column, functions and triggers have been cleaned up
+				TableMustBeCleanedUp(t, db, schema, "items", "description")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert into the new table in the new schema using its new name
+				MustInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"name":        "carrots",
+					"description": "crunchy",
+				})
+
+				// The table has the new name in the new schema and has the expected
+				// rows
+				rows := MustSelect(t, db, schema, "02_multi_operation", "products")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "description": "APPLES"},
+					{"id": 2, "name": "bananas", "description": "BANANAS"},
+					{"id": 3, "name": "carrots", "description": "crunchy"},
+				}, rows)
+
+				// The temporary column, functions and triggers have been cleaned up
+				TableMustBeCleanedUp(t, db, schema, "products", "description")
+			},
+		},
+	})
+}
+
+func TestAddColumnValidationInMultiOperationMigrations(t *testing.T) {
+	t.Parallel()
+
+	ExecuteTests(t, TestCases{
+		{
+			name: "adding a column with the same name twice fails to validate",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "items",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "serial",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "varchar(255)",
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_multi_operation",
+					Operations: migrations.Operations{
+						&migrations.OpAddColumn{
+							Table: "items",
+							Column: migrations.Column{
+								Name: "description",
+								Type: "text",
+							},
+							Up: "UPPER(name)",
+						},
+						&migrations.OpAddColumn{
+							Table: "items",
+							Column: migrations.Column{
+								Name: "description",
+								Type: "varchar(255)",
+							},
+							Up: "UPPER(name)",
+						},
+					},
+				},
+			},
+			wantStartErr: migrations.ColumnAlreadyExistsError{Table: "items", Name: "description"},
+		},
 	})
 }
 
