@@ -14,6 +14,7 @@ type OpName string
 const (
 	OpNameCreateTable               OpName = "create_table"
 	OpNameRenameTable               OpName = "rename_table"
+	OpNameRenameColumn              OpName = "rename_column"
 	OpNameDropTable                 OpName = "drop_table"
 	OpNameAddColumn                 OpName = "add_column"
 	OpNameDropColumn                OpName = "drop_column"
@@ -28,13 +29,21 @@ const (
 	OpCreateConstraintName          OpName = "create_constraint"
 )
 
-const temporaryPrefix = "_pgroll_new_"
+const (
+	temporaryPrefix = "_pgroll_new_"
+	deletedPrefix   = "_pgroll_del_"
+)
 
 // TemporaryName returns a temporary name for a given name.
 func TemporaryName(name string) string {
 	return temporaryPrefix + name
 }
 
+// DeletionName returns the deleted name for a given name.
+func DeletionName(name string) string {
+	return deletedPrefix + name
+}
+
 // ReadMigration reads a migration from an io.Reader, like a file.
 func ReadMigration(r io.Reader) (*Migration, error) {
 	byteValue, err := io.ReadAll(r)
@@ -91,6 +100,9 @@ func (v *Operations) UnmarshalJSON(data []byte) error {
 		case OpNameAddColumn:
 			item = &OpAddColumn{}
 
+		case OpNameRenameColumn:
+			item = &OpRenameColumn{}
+
 		case OpNameDropColumn:
 			item = &OpDropColumn{}
 
@@ -183,6 +195,9 @@ func OperationName(op Operation) OpName {
 	case *OpDropColumn:
 		return OpNameDropColumn
 
+	case *OpRenameColumn:
+		return OpNameRenameColumn
+
 	case *OpRenameConstraint:
 		return OpNameRenameConstraint
 
