@@ -8,26 +8,31 @@ import (
 	"errors"
 	"fmt"
 	"maps"
+	"os"
 	"slices"
+	"strconv"
+	"strings"
 	"testing"
 
 	"github.com/lib/pq"
 	"github.com/stretchr/testify/assert"
 
 	"github.com/xataio/pgroll/internal/testutils"
+	"github.com/xataio/pgroll/pkg/backfill"
 	"github.com/xataio/pgroll/pkg/migrations"
 	"github.com/xataio/pgroll/pkg/roll"
 )
 
 type TestCase struct {
-	name            string
-	migrations      []migrations.Migration
-	wantStartErr    error
-	wantRollbackErr error
-	wantCompleteErr error
-	afterStart      func(t *testing.T, db *sql.DB, schema string)
-	afterComplete   func(t *testing.T, db *sql.DB, schema string)
-	afterRollback   func(t *testing.T, db *sql.DB, schema string)
+	name              string
+	minPgMajorVersion int
+	migrations        []migrations.Migration
+	wantStartErr      error
+	wantRollbackErr   error
+	wantCompleteErr   error
+	afterStart        func(t *testing.T, db *sql.DB, schema string)
+	afterComplete     func(t *testing.T, db *sql.DB, schema string)
+	afterRollback     func(t *testing.T, db *sql.DB, schema string)
 }
 
 type TestCases []TestCase
@@ -40,13 +45,18 @@ func ExecuteTests(t *testing.T, tests TestCases, opts ...roll.Option) {
 	testSchema := testutils.TestSchema()
 
 	for _, tt := range tests {
+		if isTestSkipped(t, tt.minPgMajorVersion) {
+			t.Skipf("Skipping test %q for PostgreSQL version %s", tt.name, os.Getenv("POSTGRES_VERSION"))
+		}
+
 		t.Run(tt.name, func(t *testing.T) {
 			testutils.WithMigratorInSchemaAndConnectionToContainerWithOptions(t, testSchema, opts, func(mig *roll.Roll, db *sql.DB) {
 				ctx := context.Background()
+				config := backfill.NewConfig()
 
 				// run all migrations except the last one
 				for i := 0; i < len(tt.migrations)-1; i++ {
-					if err := mig.Start(ctx, &tt.migrations[i]); err != nil {
+					if err := mig.Start(ctx, &tt.migrations[i], config); err != nil {
 						t.Fatalf("Failed to start migration: %v", err)
 					}
 
@@ -56,7 +66,7 @@ func ExecuteTests(t *testing.T, tests TestCases, opts ...roll.Option) {
 				}
 
 				// start the last migration
-				err := mig.Start(ctx, &tt.migrations[len(tt.migrations)-1])
+				err := mig.Start(ctx, &tt.migrations[len(tt.migrations)-1], config)
 				if tt.wantStartErr != nil {
 					if !errors.Is(err, tt.wantStartErr) {
 						t.Fatalf("Expected error %q, got %q", tt.wantStartErr, err)
@@ -90,7 +100,7 @@ func ExecuteTests(t *testing.T, tests TestCases, opts ...roll.Option) {
 				}
 
 				// re-start the last migration
-				if err := mig.Start(ctx, &tt.migrations[len(tt.migrations)-1]); err != nil {
+				if err := mig.Start(ctx, &tt.migrations[len(tt.migrations)-1], config); err != nil {
 					t.Fatalf("Failed to start migration: %v", err)
 				}
 
@@ -217,14 +227,21 @@ func TriggerMustNotExist(t *testing.T, db *sql.DB, schema, table, trigger string
 
 func CheckConstraintMustNotExist(t *testing.T, db *sql.DB, schema, table, constraint string) {
 	t.Helper()
-	if checkConstraintExists(t, db, schema, table, constraint) {
+	if checkConstraintExists(t, db, schema, table, constraint, false) {
 		t.Fatalf("Expected constraint %q to not exist", constraint)
 	}
 }
 
 func CheckConstraintMustExist(t *testing.T, db *sql.DB, schema, table, constraint string) {
 	t.Helper()
-	if !checkConstraintExists(t, db, schema, table, constraint) {
+	if !checkConstraintExists(t, db, schema, table, constraint, false) {
+		t.Fatalf("Expected constraint %q to exist", constraint)
+	}
+}
+
+func NotInheritableCheckConstraintMustExist(t *testing.T, db *sql.DB, schema, table, constraint string) {
+	t.Helper()
+	if !checkConstraintExists(t, db, schema, table, constraint, true) {
 		t.Fatalf("Expected constraint %q to exist", constraint)
 	}
 }
@@ -236,20 +253,55 @@ func UniqueConstraintMustExist(t *testing.T, db *sql.DB, schema, table, constrai
 	}
 }
 
-func ValidatedForeignKeyMustExist(t *testing.T, db *sql.DB, schema, table, constraint string, opts ...foreignKeyOnDeleteOpt) {
+func ValidatedForeignKeyMustExist(t *testing.T, db *sql.DB, schema, table, constraint string) {
 	t.Helper()
-	if !foreignKeyExists(t, db, schema, table, constraint, true, opts...) {
+	if !foreignKeyExists(t, db, schema, table, constraint, true, migrations.ForeignKeyActionNOACTION, migrations.ForeignKeyActionNOACTION) {
 		t.Fatalf("Expected validated foreign key %q to exist", constraint)
 	}
 }
 
-func NotValidatedForeignKeyMustExist(t *testing.T, db *sql.DB, schema, table, constraint string, opts ...foreignKeyOnDeleteOpt) {
+func ValidatedForeignKeyMustExistWithReferentialAction(t *testing.T, db *sql.DB, schema, table, constraint string, onDelete, onUpdate migrations.ForeignKeyAction) {
 	t.Helper()
-	if !foreignKeyExists(t, db, schema, table, constraint, false, opts...) {
+	if !foreignKeyExists(t, db, schema, table, constraint, true, onDelete, onUpdate) {
+		t.Fatalf("Expected validated foreign key %q to exist", constraint)
+	}
+}
+
+func NotValidatedForeignKeyMustExist(t *testing.T, db *sql.DB, schema, table, constraint string) {
+	t.Helper()
+	if !foreignKeyExists(t, db, schema, table, constraint, false, migrations.ForeignKeyActionNOACTION, migrations.ForeignKeyActionNOACTION) {
 		t.Fatalf("Expected not validated foreign key %q to exist", constraint)
 	}
 }
 
+func NotValidatedForeignKeyMustExistWithReferentialAction(t *testing.T, db *sql.DB, schema, table, constraint string, onDelete, onUpdate migrations.ForeignKeyAction) {
+	t.Helper()
+	if !foreignKeyExists(t, db, schema, table, constraint, false, onDelete, onUpdate) {
+		t.Fatalf("Expected not validated foreign key %q to exist", constraint)
+	}
+}
+
+func TableForeignKeyMustExist(t *testing.T, db *sql.DB, schema, table, constraint string, deferrable, initiallyDeferred bool) {
+	t.Helper()
+	if !tableForeignKeyExists(t, db, schema, table, constraint, deferrable, initiallyDeferred) {
+		t.Fatalf("Expected table foreign key %q to exist", constraint)
+	}
+}
+
+func PrimaryKeyConstraintMustExist(t *testing.T, db *sql.DB, schema, table, constraint string) {
+	t.Helper()
+	if !primaryKeyConstraintExists(t, db, schema, table, constraint) {
+		t.Fatalf("Expected constraint %q to exist", constraint)
+	}
+}
+
+func ExcludeConstraintMustExist(t *testing.T, db *sql.DB, schema, table, constraint string) {
+	t.Helper()
+	if !excludeConstraintExists(t, db, schema, table, constraint) {
+		t.Fatalf("Expected constraint %q to exist", constraint)
+	}
+}
+
 func IndexMustExist(t *testing.T, db *sql.DB, schema, table, index string) {
 	t.Helper()
 	if !indexExists(t, db, schema, table, index) {
@@ -257,6 +309,13 @@ func IndexMustExist(t *testing.T, db *sql.DB, schema, table, index string) {
 	}
 }
 
+func IndexDescendingMustExist(t *testing.T, db *sql.DB, schema, table, index string, columnIdx int) {
+	t.Helper()
+	if !indexDescendingExists(t, db, schema, table, index, columnIdx) {
+		t.Fatalf("Expected index %q to exist", index)
+	}
+}
+
 func IndexMustNotExist(t *testing.T, db *sql.DB, schema, table, index string) {
 	t.Helper()
 	if indexExists(t, db, schema, table, index) {
@@ -305,6 +364,25 @@ func indexExists(t *testing.T, db *sql.DB, schema, table, index string) bool {
 	return exists
 }
 
+func indexDescendingExists(t *testing.T, db *sql.DB, schema, table, index string, columnIdx int) bool {
+	t.Helper()
+
+	var flags []uint8
+	err := db.QueryRow(`
+    SELECT indoption
+    FROM pg_index
+    WHERE indrelid = $1::regclass
+    AND indexrelid = $2::regclass`,
+		fmt.Sprintf("%s.%s", schema, table), fmt.Sprintf("%s.%s", schema, index)).Scan(&flags)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	// check if index is descending using the 1st bit of the flags
+	indoptionDesc := uint8(1)
+	return flags[columnIdx]&indoptionDesc == 1
+}
+
 func CheckIndexDefinition(t *testing.T, db *sql.DB, schema, table, index, expectedDefinition string) {
 	t.Helper()
 
@@ -326,7 +404,7 @@ func CheckIndexDefinition(t *testing.T, db *sql.DB, schema, table, index, expect
 	}
 }
 
-func checkConstraintExists(t *testing.T, db *sql.DB, schema, table, constraint string) bool {
+func checkConstraintExists(t *testing.T, db *sql.DB, schema, table, constraint string, noInherit bool) bool {
 	t.Helper()
 
 	var exists bool
@@ -337,8 +415,9 @@ func checkConstraintExists(t *testing.T, db *sql.DB, schema, table, constraint s
       WHERE conrelid = $1::regclass
       AND conname = $2
       AND contype = 'c'
+      AND connoinherit = $3
     )`,
-		fmt.Sprintf("%s.%s", schema, table), constraint).Scan(&exists)
+		fmt.Sprintf("%s.%s", schema, table), constraint, noInherit).Scan(&exists)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -366,23 +445,25 @@ func uniqueConstraintExists(t *testing.T, db *sql.DB, schema, table, constraint
 	return exists
 }
 
-type foreignKeyOnDeleteOpt func() string
-
-func withOnDeleteCascade() foreignKeyOnDeleteOpt {
-	return func() string { return "c" }
-}
-
-func withOnDeleteSetNull() foreignKeyOnDeleteOpt {
-	return func() string { return "n" }
-}
-
-func foreignKeyExists(t *testing.T, db *sql.DB, schema, table, constraint string, validated bool, opts ...foreignKeyOnDeleteOpt) bool {
-	t.Helper()
-
-	confDelType := "a"
-	for _, opt := range opts {
-		confDelType = opt()
+func referentialAction(a migrations.ForeignKeyAction) string {
+	switch a {
+	case migrations.ForeignKeyActionNOACTION:
+		return "a"
+	case migrations.ForeignKeyActionRESTRICT:
+		return "r"
+	case migrations.ForeignKeyActionSETNULL:
+		return "n"
+	case migrations.ForeignKeyActionSETDEFAULT:
+		return "d"
+	case migrations.ForeignKeyActionCASCADE:
+		return "c"
+	default:
+		return "a"
 	}
+}
+
+func foreignKeyExists(t *testing.T, db *sql.DB, schema, table, constraint string, validated bool, onDeleteAction, onUpdateAction migrations.ForeignKeyAction) bool {
+	t.Helper()
 
 	var exists bool
 	err := db.QueryRow(`
@@ -393,9 +474,82 @@ func foreignKeyExists(t *testing.T, db *sql.DB, schema, table, constraint string
       AND conname = $2
       AND contype = 'f'
       AND convalidated = $3
-      AND confdeltype = $4 
+      AND confdeltype = $4
+      AND confupdtype = $5
     )`,
-		fmt.Sprintf("%s.%s", schema, table), constraint, validated, confDelType).Scan(&exists)
+		fmt.Sprintf("%s.%s", schema, table), constraint, validated, referentialAction(onDeleteAction), referentialAction(onUpdateAction)).Scan(&exists)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	return exists
+}
+
+func tableForeignKeyExists(t *testing.T, db *sql.DB, schema, table, constraint string, deferrable, initiallyDeferred bool) bool {
+	t.Helper()
+
+	deferrableStr := "NO"
+	if deferrable {
+		deferrableStr = "YES"
+	}
+	initiallyDeferredStr := "NO"
+	if initiallyDeferred {
+		initiallyDeferredStr = "YES"
+	}
+
+	var exists bool
+	err := db.QueryRow(`
+    SELECT EXISTS (
+      SELECT 1
+      FROM information_schema.table_constraints
+      WHERE table_schema = $1
+      AND table_name = $2
+      AND constraint_name = $3
+      AND constraint_type = 'FOREIGN KEY'
+      AND is_deferrable = $4
+      AND initially_deferred = $5
+    )`,
+		schema, table, constraint, deferrableStr, initiallyDeferredStr).Scan(&exists)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	return exists
+}
+
+func primaryKeyConstraintExists(t *testing.T, db *sql.DB, schema, table, constraint string) bool {
+	t.Helper()
+
+	var exists bool
+	err := db.QueryRow(`
+    SELECT EXISTS (
+      SELECT 1
+      FROM pg_catalog.pg_constraint
+      WHERE conrelid = $1::regclass
+      AND conname = $2
+      AND contype = 'p'
+    )`,
+		fmt.Sprintf("%s.%s", schema, table), constraint).Scan(&exists)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	return exists
+}
+
+func excludeConstraintExists(t *testing.T, db *sql.DB, schema, table, constraint string) bool {
+	t.Helper()
+
+	var exists bool
+	err := db.QueryRow(`
+    SELECT EXISTS (
+      SELECT 1
+      FROM pg_catalog.pg_constraint
+      WHERE conrelid = $1::regclass
+      AND conname = $2
+      AND contype = 'x'
+    )`,
+		fmt.Sprintf("%s.%s", schema, table), constraint).Scan(&exists)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -754,6 +908,7 @@ func MustSelect(t *testing.T, db *sql.DB, schema, version, table string) []map[s
 // - The down functions for the columns no longer exist.
 // - The up triggers for the columns no longer exist.
 // - The down triggers for the columns no longer exist.
+// - The _pgroll_needs_backfill column should not exist on the table.
 func TableMustBeCleanedUp(t *testing.T, db *sql.DB, schema, table string, columns ...string) {
 	t.Helper()
 
@@ -761,6 +916,9 @@ func TableMustBeCleanedUp(t *testing.T, db *sql.DB, schema, table string, column
 		// The temporary column should not exist on the underlying table.
 		ColumnMustNotExist(t, db, schema, table, migrations.TemporaryName(column))
 
+		// The _pgroll_needs_backfill column should not exist on the table.
+		ColumnMustNotExist(t, db, schema, table, migrations.CNeedsBackfillColumn)
+
 		// The up function for the column no longer exists.
 		FunctionMustNotExist(t, db, schema, migrations.TriggerFunctionName(table, column))
 		// The down function for the column no longer exists.
@@ -782,4 +940,16 @@ func mustSetSearchPath(t *testing.T, db *sql.DB, schema string) {
 	}
 }
 
+func isTestSkipped(t *testing.T, minPgMajorVersion int) bool {
+	if minPgMajorVersion == 0 || os.Getenv("POSTGRES_VERSION") == "" || os.Getenv("POSTGRES_VERSION") == "latest" {
+		return false
+	}
+	pgMajorVersion := strings.Split(os.Getenv("POSTGRES_VERSION"), ".")[0]
+	version, err := strconv.Atoi(pgMajorVersion)
+	if err != nil {
+		return false
+	}
+	return version < minPgMajorVersion
+}
+
 func ptr[T any](x T) *T { return &x }
