@@ -14,12 +14,18 @@ import (
 
 var _ Operation = (*OpDropConstraint)(nil)
 
-func (o *OpDropConstraint) Start(ctx context.Context, conn db.DB, latestSchema string, tr SQLTransformer, s *schema.Schema, cbs ...CallbackFn) (*schema.Table, error) {
+func (o *OpDropConstraint) Start(ctx context.Context, conn db.DB, latestSchema string, s *schema.Schema) (*schema.Table, error) {
 	table := s.GetTable(o.Table)
+	if table == nil {
+		return nil, TableDoesNotExistError{Name: o.Table}
+	}
 
 	// By this point Validate() should have run which ensures the constraint exists and that we only have
 	// one column associated with it.
 	column := table.GetColumn(table.GetConstraintColumns(o.Name)[0])
+	if column == nil {
+		return nil, ColumnDoesNotExistError{Table: o.Table, Name: table.GetConstraintColumns(o.Name)[0]}
+	}
 
 	// Create a copy of the column on the underlying table.
 	d := NewColumnDuplicator(conn, table, column).WithoutConstraint(o.Name)
@@ -28,7 +34,7 @@ func (o *OpDropConstraint) Start(ctx context.Context, conn db.DB, latestSchema s
 	}
 
 	// Add a trigger to copy values from the old column to the new, rewriting values using the `up` SQL.
-	err := createTrigger(ctx, conn, tr, triggerConfig{
+	err := createTrigger(ctx, conn, triggerConfig{
 		Name:           TriggerName(o.Table, column.Name),
 		Direction:      TriggerDirectionUp,
 		Columns:        table.Columns,
@@ -45,12 +51,12 @@ func (o *OpDropConstraint) Start(ctx context.Context, conn db.DB, latestSchema s
 	// Add the new column to the internal schema representation. This is done
 	// here, before creation of the down trigger, so that the trigger can declare
 	// a variable for the new column.
-	table.AddColumn(column.Name, schema.Column{
+	table.AddColumn(column.Name, &schema.Column{
 		Name: TemporaryName(column.Name),
 	})
 
 	// Add a trigger to copy values from the new column to the old, rewriting values using the `down` SQL.
-	err = createTrigger(ctx, conn, tr, triggerConfig{
+	err = createTrigger(ctx, conn, triggerConfig{
 		Name:           TriggerName(o.Table, TemporaryName(column.Name)),
 		Direction:      TriggerDirectionDown,
 		Columns:        table.Columns,
@@ -66,7 +72,7 @@ func (o *OpDropConstraint) Start(ctx context.Context, conn db.DB, latestSchema s
 	return table, nil
 }
 
-func (o *OpDropConstraint) Complete(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
+func (o *OpDropConstraint) Complete(ctx context.Context, conn db.DB, s *schema.Schema) error {
 	// We have already validated that there is single column related to this constraint.
 	table := s.GetTable(o.Table)
 	column := table.GetColumn(table.GetConstraintColumns(o.Name)[0])
@@ -85,6 +91,18 @@ func (o *OpDropConstraint) Complete(ctx context.Context, conn db.DB, tr SQLTrans
 		return err
 	}
 
+	if err := alterSequenceOwnerToDuplicatedColumn(ctx, conn, o.Table, column.Name); err != nil {
+		return err
+	}
+
+	// Remove the needs backfill column
+	_, err = conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE IF EXISTS %s DROP COLUMN IF EXISTS %s",
+		pq.QuoteIdentifier(table.Name),
+		pq.QuoteIdentifier(CNeedsBackfillColumn)))
+	if err != nil {
+		return err
+	}
+
 	// Drop the old column
 	_, err = conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE IF EXISTS %s DROP COLUMN IF EXISTS %s",
 		pq.QuoteIdentifier(o.Table),
@@ -101,15 +119,15 @@ func (o *OpDropConstraint) Complete(ctx context.Context, conn db.DB, tr SQLTrans
 	return err
 }
 
-func (o *OpDropConstraint) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
+func (o *OpDropConstraint) Rollback(ctx context.Context, conn db.DB, s *schema.Schema) error {
 	// We have already validated that there is single column related to this constraint.
 	table := s.GetTable(o.Table)
-	column := table.GetColumn(table.GetConstraintColumns(o.Name)[0])
+	columnName := table.GetConstraintColumns(o.Name)[0]
 
 	// Drop the new column
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE %s DROP COLUMN IF EXISTS %s",
 		pq.QuoteIdentifier(o.Table),
-		pq.QuoteIdentifier(TemporaryName(column.Name)),
+		pq.QuoteIdentifier(TemporaryName(columnName)),
 	))
 	if err != nil {
 		return err
@@ -117,7 +135,7 @@ func (o *OpDropConstraint) Rollback(ctx context.Context, conn db.DB, tr SQLTrans
 
 	// Remove the up function and trigger
 	_, err = conn.ExecContext(ctx, fmt.Sprintf("DROP FUNCTION IF EXISTS %s CASCADE",
-		pq.QuoteIdentifier(TriggerFunctionName(o.Table, column.Name)),
+		pq.QuoteIdentifier(TriggerFunctionName(o.Table, columnName)),
 	))
 	if err != nil {
 		return err
@@ -125,8 +143,16 @@ func (o *OpDropConstraint) Rollback(ctx context.Context, conn db.DB, tr SQLTrans
 
 	// Remove the down function and trigger
 	_, err = conn.ExecContext(ctx, fmt.Sprintf("DROP FUNCTION IF EXISTS %s CASCADE",
-		pq.QuoteIdentifier(TriggerFunctionName(o.Table, TemporaryName(column.Name))),
+		pq.QuoteIdentifier(TriggerFunctionName(o.Table, TemporaryName(columnName))),
 	))
+	if err != nil {
+		return err
+	}
+
+	// Remove the needs backfill column
+	_, err = conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE IF EXISTS %s DROP COLUMN IF EXISTS %s",
+		pq.QuoteIdentifier(table.Name),
+		pq.QuoteIdentifier(CNeedsBackfillColumn)))
 
 	return err
 }
