@@ -369,7 +369,7 @@ func TestDropConstraint(t *testing.T) {
 					Operations: migrations.Operations{
 						&migrations.OpDropConstraint{
 							Table: "users",
-							Name:  "_pgroll_new_users_name_key",
+							Name:  "users_name_key",
 							Up:    "name",
 							Down:  "name || '-' || (random()*1000000)::integer",
 						},
@@ -409,6 +409,69 @@ func TestDropConstraint(t *testing.T) {
 				})
 			},
 		},
+		{
+			name: "drop unique constraint from serial column",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_add_tables",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "users",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "serial",
+									Pk:   true,
+								},
+								{
+									Name:   "secondary_id",
+									Type:   "serial",
+									Unique: true,
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_drop_unique_constraint",
+					Operations: migrations.Operations{
+						&migrations.OpDropConstraint{
+							Table: "users",
+							Name:  "users_secondary_id_key",
+							Up:    "secondary_id",
+							Down:  "secondary_id",
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// The new (temporary) `secondary_id` column should exist on the underlying table.
+				ColumnMustExist(t, db, schema, "users", migrations.TemporaryName("secondary_id"))
+
+				// Inserting a row that meets the unique constraint into the old view works.
+				MustInsert(t, db, schema, "01_add_tables", "users", map[string]string{
+					"secondary_id": "1",
+				})
+
+				// Inserting a row that does not meet the unique constraint into the old view fails.
+				MustNotInsert(t, db, schema, "01_add_tables", "users", map[string]string{
+					"secondary_id": "1",
+				}, testutils.UniqueViolationErrorCode)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table is cleaned up; temporary columns, trigger functions and triggers no longer exist.
+				TableMustBeCleanedUp(t, db, schema, "users", "secondary_id")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// The table is cleaned up; temporary columns, trigger functions and triggers no longer exist.
+				TableMustBeCleanedUp(t, db, schema, "users", "secondary_id")
+
+				// Inserting a row that does not meet the unique constraint into the new view works.
+				MustInsert(t, db, schema, "02_drop_unique_constraint", "users", map[string]string{
+					"secondary_id": "1",
+				})
+			},
+		},
 		{
 			name: "dropping a unique constraint preserves the column's default value",
 			migrations: []migrations.Migration{
@@ -438,7 +501,7 @@ func TestDropConstraint(t *testing.T) {
 					Operations: migrations.Operations{
 						&migrations.OpDropConstraint{
 							Table: "users",
-							Name:  "_pgroll_new_users_name_key",
+							Name:  "users_name_key",
 							Up:    "name",
 							Down:  "name || '-' || (random()*1000000)::integer",
 						},
@@ -537,7 +600,7 @@ func TestDropConstraint(t *testing.T) {
 					Operations: migrations.Operations{
 						&migrations.OpDropConstraint{
 							Table: "employees",
-							Name:  "_pgroll_new_employees_department_id_key",
+							Name:  "employees_department_id_key",
 							Up:    "department_id",
 							Down:  "department_id",
 						},
@@ -599,7 +662,7 @@ func TestDropConstraint(t *testing.T) {
 					Operations: migrations.Operations{
 						&migrations.OpDropConstraint{
 							Table: "posts",
-							Name:  "_pgroll_new_posts_title_key",
+							Name:  "posts_title_key",
 							Up:    "title",
 							Down:  "title",
 						},
@@ -742,7 +805,7 @@ func TestDropConstraint(t *testing.T) {
 					Operations: migrations.Operations{
 						&migrations.OpDropConstraint{
 							Table: "posts",
-							Name:  "_pgroll_new_posts_title_key",
+							Name:  "posts_title_key",
 							Up:    "title",
 							Down:  "title",
 						},
@@ -794,7 +857,7 @@ func TestDropConstraint(t *testing.T) {
 					Operations: migrations.Operations{
 						&migrations.OpDropConstraint{
 							Table: "posts",
-							Name:  "_pgroll_new_posts_title_key",
+							Name:  "posts_title_key",
 							Up:    "title",
 							Down:  "title",
 						},
