@@ -25,12 +25,12 @@ func TestDropIndex(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "name",
 									Type:     "varchar(255)",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 							},
 						},
@@ -42,7 +42,7 @@ func TestDropIndex(t *testing.T) {
 						&migrations.OpCreateIndex{
 							Name:    "idx_users_name",
 							Table:   "users",
-							Columns: []string{"name"},
+							Columns: map[string]migrations.IndexField{"name": {}},
 						},
 					},
 				},
@@ -79,12 +79,12 @@ func TestDropIndex(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "name",
 									Type:     "varchar(255)",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 							},
 						},
@@ -96,7 +96,7 @@ func TestDropIndex(t *testing.T) {
 						&migrations.OpCreateIndex{
 							Name:    "idx_USERS_name",
 							Table:   "users",
-							Columns: []string{"name"},
+							Columns: map[string]migrations.IndexField{"name": {}},
 						},
 					},
 				},
