@@ -14,18 +14,20 @@ import (
 
 var _ Operation = (*OpRenameTable)(nil)
 
-func (o *OpRenameTable) Start(ctx context.Context, conn db.DB, latestSchema string, tr SQLTransformer, s *schema.Schema, cbs ...CallbackFn) (*schema.Table, error) {
+func (o *OpRenameTable) Start(ctx context.Context, conn db.DB, latestSchema string, s *schema.Schema) (*schema.Table, error) {
 	return nil, s.RenameTable(o.From, o.To)
 }
 
-func (o *OpRenameTable) Complete(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
+func (o *OpRenameTable) Complete(ctx context.Context, conn db.DB, s *schema.Schema) error {
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE IF EXISTS %s RENAME TO %s",
 		pq.QuoteIdentifier(o.From),
 		pq.QuoteIdentifier(o.To)))
+
 	return err
 }
 
-func (o *OpRenameTable) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
+func (o *OpRenameTable) Rollback(ctx context.Context, conn db.DB, s *schema.Schema) error {
+	s.RenameTable(o.To, o.From)
 	return nil
 }
 
@@ -40,5 +42,6 @@ func (o *OpRenameTable) Validate(ctx context.Context, s *schema.Schema) error {
 		return err
 	}
 
+	s.RenameTable(o.From, o.To)
 	return nil
 }
