@@ -47,6 +47,7 @@ func TestSetCheckConstraint(t *testing.T) {
 							Check: &migrations.CheckConstraint{
 								Name:       "check_title_length",
 								Constraint: "length(title) > 3",
+								NoInherit:  true,
 							},
 							Up:   "SELECT CASE WHEN length(title) <= 3 THEN LPAD(title, 4, '-') ELSE title END",
 							Down: "title",
@@ -59,7 +60,7 @@ func TestSetCheckConstraint(t *testing.T) {
 				ColumnMustExist(t, db, schema, "posts", migrations.TemporaryName("title"))
 
 				// A check constraint has been added to the temporary column
-				CheckConstraintMustExist(t, db, schema, "posts", "check_title_length")
+				NotInheritableCheckConstraintMustExist(t, db, schema, "posts", "check_title_length")
 
 				// Inserting a row that meets the check constraint into the old view works.
 				MustInsert(t, db, schema, "01_add_table", "posts", map[string]string{
@@ -103,7 +104,7 @@ func TestSetCheckConstraint(t *testing.T) {
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
 				// The check constraint exists on the new table.
-				CheckConstraintMustExist(t, db, schema, "posts", "check_title_length")
+				NotInheritableCheckConstraintMustExist(t, db, schema, "posts", "check_title_length")
 
 				// Inserting a row that meets the check constraint into the new view works.
 				MustInsert(t, db, schema, "02_add_check_constraint", "posts", map[string]string{
@@ -249,7 +250,7 @@ func TestSetCheckConstraint(t *testing.T) {
 										Name:     "fk_employee_department",
 										Table:    "departments",
 										Column:   "id",
-										OnDelete: migrations.ForeignKeyReferenceOnDeleteCASCADE,
+										OnDelete: migrations.ForeignKeyActionCASCADE,
 									},
 								},
 							},
@@ -274,13 +275,27 @@ func TestSetCheckConstraint(t *testing.T) {
 			},
 			afterStart: func(t *testing.T, db *sql.DB, schema string) {
 				// A temporary FK constraint has been created on the temporary column
-				ValidatedForeignKeyMustExist(t, db, schema, "employees", migrations.DuplicationName("fk_employee_department"), withOnDeleteCascade())
+				ValidatedForeignKeyMustExistWithReferentialAction(
+					t,
+					db,
+					schema,
+					"employees",
+					migrations.DuplicationName("fk_employee_department"),
+					migrations.ForeignKeyActionCASCADE,
+					migrations.ForeignKeyActionNOACTION)
 			},
 			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
 				// The foreign key constraint still exists on the column
-				ValidatedForeignKeyMustExist(t, db, schema, "employees", "fk_employee_department", withOnDeleteCascade())
+				ValidatedForeignKeyMustExistWithReferentialAction(
+					t,
+					db,
+					schema,
+					"employees",
+					"fk_employee_department",
+					migrations.ForeignKeyActionCASCADE,
+					migrations.ForeignKeyActionNOACTION)
 			},
 		},
 		{
@@ -594,6 +609,222 @@ func TestSetCheckConstraint(t *testing.T) {
 	})
 }
 
+func TestSetCheckInMultiOperationMigrations(t *testing.T) {
+	t.Parallel()
+
+	ExecuteTests(t, TestCases{
+		{
+			name: "rename table, set check constraint",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "items",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name:     "name",
+									Type:     "varchar(255)",
+									Nullable: true,
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_multi_operation",
+					Operations: migrations.Operations{
+						&migrations.OpRenameTable{
+							From: "items",
+							To:   "products",
+						},
+						&migrations.OpAlterColumn{
+							Table:  "products",
+							Column: "name",
+							Check: &migrations.CheckConstraint{
+								Name:       "check_name_length",
+								Constraint: "LENGTH(name) > 2",
+							},
+							Up:   "SELECT CASE WHEN length(name) > 2 THEN name ELSE name || '---' END",
+							Down: "name",
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the new view that meets the check constraint
+				MustInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":   "1",
+					"name": "abc",
+				})
+
+				// Can't insert a row into the new view that violates the check constraint
+				MustNotInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":   "2",
+					"name": "x",
+				}, testutils.CheckViolationErrorCode)
+
+				// Can insert a row into the old view that violates the check constraint
+				MustInsert(t, db, schema, "01_create_table", "items", map[string]string{
+					"id":   "3",
+					"name": "x",
+				})
+
+				// The new view has the expected rows
+				rows := MustSelect(t, db, schema, "02_multi_operation", "products")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "abc"},
+					{"id": 3, "name": "x---"},
+				}, rows)
+
+				// The old view has the expected rows
+				rows = MustSelect(t, db, schema, "01_create_table", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "abc"},
+					{"id": 3, "name": "x"},
+				}, rows)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table has been cleaned up
+				TableMustBeCleanedUp(t, db, schema, "items", "name")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the new view that meets the check constraint
+				MustInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":   "4",
+					"name": "def",
+				})
+
+				// Can't insert a row into the new view that violates the check constraint
+				MustNotInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":   "5",
+					"name": "x",
+				}, testutils.CheckViolationErrorCode)
+
+				// The new view has the expected rows
+				rows := MustSelect(t, db, schema, "02_multi_operation", "products")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "abc"},
+					{"id": 3, "name": "x---"},
+					{"id": 4, "name": "def"},
+				}, rows)
+			},
+		},
+		{
+			name: "rename table, rename column set check constraint",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "items",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name:     "name",
+									Type:     "varchar(255)",
+									Nullable: true,
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_multi_operation",
+					Operations: migrations.Operations{
+						&migrations.OpRenameTable{
+							From: "items",
+							To:   "products",
+						},
+						&migrations.OpRenameColumn{
+							Table: "products",
+							From:  "name",
+							To:    "item_name",
+						},
+						&migrations.OpAlterColumn{
+							Table:  "products",
+							Column: "item_name",
+							Check: &migrations.CheckConstraint{
+								Name:       "check_name_length",
+								Constraint: "LENGTH(item_name) > 2",
+							},
+							Up:   "SELECT CASE WHEN length(item_name) > 2 THEN item_name ELSE item_name || '---' END",
+							Down: "item_name",
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the new view that meets the check constraint
+				MustInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":        "1",
+					"item_name": "abc",
+				})
+
+				// Can't insert a row into the new view that violates the check constraint
+				MustNotInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":        "2",
+					"item_name": "x",
+				}, testutils.CheckViolationErrorCode)
+
+				// Can insert a row into the old view that violates the check constraint
+				MustInsert(t, db, schema, "01_create_table", "items", map[string]string{
+					"id":   "3",
+					"name": "x",
+				})
+
+				// The new view has the expected rows
+				rows := MustSelect(t, db, schema, "02_multi_operation", "products")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "item_name": "abc"},
+					{"id": 3, "item_name": "x---"},
+				}, rows)
+
+				// The old view has the expected rows
+				rows = MustSelect(t, db, schema, "01_create_table", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "abc"},
+					{"id": 3, "name": "x"},
+				}, rows)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table has been cleaned up
+				TableMustBeCleanedUp(t, db, schema, "items", "name")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the new view that meets the check constraint
+				MustInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":        "4",
+					"item_name": "def",
+				})
+
+				// Can't insert a row into the new view that violates the check constraint
+				MustNotInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":        "5",
+					"item_name": "x",
+				}, testutils.CheckViolationErrorCode)
+
+				// The new view has the expected rows
+				rows := MustSelect(t, db, schema, "02_multi_operation", "products")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "item_name": "abc"},
+					{"id": 3, "item_name": "x---"},
+					{"id": 4, "item_name": "def"},
+				}, rows)
+			},
+		},
+	})
+}
+
 func TestSetCheckConstraintValidation(t *testing.T) {
 	t.Parallel()
 
