@@ -22,32 +22,39 @@ type OpSetDefault struct {
 
 var _ Operation = (*OpSetDefault)(nil)
 
-func (o *OpSetDefault) Start(ctx context.Context, conn db.DB, latestSchema string, tr SQLTransformer, s *schema.Schema, cbs ...CallbackFn) (*schema.Table, error) {
-	tbl := s.GetTable(o.Table)
+func (o *OpSetDefault) Start(ctx context.Context, conn db.DB, latestSchema string, s *schema.Schema) (*schema.Table, error) {
+	table := s.GetTable(o.Table)
+	if table == nil {
+		return nil, TableDoesNotExistError{Name: o.Table}
+	}
+	column := table.GetColumn(o.Column)
+	if column == nil {
+		return nil, ColumnDoesNotExistError{Table: o.Table, Name: o.Column}
+	}
 
 	var err error
 	if o.Default == nil {
 		_, err = conn.ExecContext(ctx, fmt.Sprintf(`ALTER TABLE %s ALTER COLUMN %s DROP DEFAULT`,
-			pq.QuoteIdentifier(o.Table),
-			pq.QuoteIdentifier(TemporaryName(o.Column))))
+			pq.QuoteIdentifier(table.Name),
+			pq.QuoteIdentifier(column.Name)))
 	} else {
 		_, err = conn.ExecContext(ctx, fmt.Sprintf(`ALTER TABLE %s ALTER COLUMN %s SET DEFAULT %s`,
-			pq.QuoteIdentifier(o.Table),
-			pq.QuoteIdentifier(TemporaryName(o.Column)),
+			pq.QuoteIdentifier(table.Name),
+			pq.QuoteIdentifier(column.Name),
 			*o.Default))
 	}
 	if err != nil {
 		return nil, err
 	}
 
-	return tbl, nil
+	return table, nil
 }
 
-func (o *OpSetDefault) Complete(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
+func (o *OpSetDefault) Complete(ctx context.Context, conn db.DB, s *schema.Schema) error {
 	return nil
 }
 
-func (o *OpSetDefault) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
+func (o *OpSetDefault) Rollback(ctx context.Context, conn db.DB, s *schema.Schema) error {
 	return nil
 }
 
