@@ -304,7 +304,7 @@ func TestSetForeignKey(t *testing.T) {
 								Name:     "fk_users_id",
 								Table:    "users",
 								Column:   "id",
-								OnDelete: migrations.ForeignKeyReferenceOnDeleteCASCADE,
+								OnDelete: migrations.ForeignKeyActionCASCADE,
 							},
 							Up:   "SELECT CASE WHEN EXISTS (SELECT 1 FROM users WHERE users.id = user_id) THEN user_id ELSE NULL END",
 							Down: "user_id",
@@ -410,7 +410,7 @@ func TestSetForeignKey(t *testing.T) {
 								Name:     "fk_users_id",
 								Table:    "users",
 								Column:   "id",
-								OnDelete: migrations.ForeignKeyReferenceOnDeleteSETNULL,
+								OnDelete: migrations.ForeignKeyActionSETNULL,
 							},
 							Up:   "SELECT CASE WHEN EXISTS (SELECT 1 FROM users WHERE users.id = user_id) THEN user_id ELSE NULL END",
 							Down: "user_id",
@@ -520,7 +520,7 @@ func TestSetForeignKey(t *testing.T) {
 								Name:     "fk_users_id",
 								Table:    "users",
 								Column:   "id",
-								OnDelete: migrations.ForeignKeyReferenceOnDeleteSETDEFAULT,
+								OnDelete: migrations.ForeignKeyActionSETDEFAULT,
 							},
 							Up:   "SELECT CASE WHEN EXISTS (SELECT 1 FROM users WHERE users.id = user_id) THEN user_id ELSE NULL END",
 							Down: "user_id",
@@ -725,7 +725,7 @@ func TestSetForeignKey(t *testing.T) {
 								Name:     "fk_users_id_1",
 								Table:    "users",
 								Column:   "id",
-								OnDelete: migrations.ForeignKeyReferenceOnDeleteCASCADE,
+								OnDelete: migrations.ForeignKeyActionCASCADE,
 							},
 							Up:   "SELECT CASE WHEN EXISTS (SELECT 1 FROM users WHERE users.id = user_id) THEN user_id ELSE NULL END",
 							Down: "user_id",
@@ -751,13 +751,27 @@ func TestSetForeignKey(t *testing.T) {
 			},
 			afterStart: func(t *testing.T, db *sql.DB, schema string) {
 				// A temporary FK constraint has been created on the temporary column
-				ValidatedForeignKeyMustExist(t, db, schema, "posts", migrations.DuplicationName("fk_users_id_1"), withOnDeleteCascade())
+				ValidatedForeignKeyMustExistWithReferentialAction(
+					t,
+					db,
+					schema,
+					"posts",
+					migrations.DuplicationName("fk_users_id_1"),
+					migrations.ForeignKeyActionCASCADE,
+					migrations.ForeignKeyActionNOACTION)
 			},
 			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
 				// The foreign key constraint still exists on the column
-				ValidatedForeignKeyMustExist(t, db, schema, "posts", "fk_users_id_1", withOnDeleteCascade())
+				ValidatedForeignKeyMustExistWithReferentialAction(
+					t,
+					db,
+					schema,
+					"posts",
+					"fk_users_id_1",
+					migrations.ForeignKeyActionCASCADE,
+					migrations.ForeignKeyActionNOACTION)
 			},
 		},
 		{
@@ -1188,6 +1202,549 @@ func TestSetForeignKey(t *testing.T) {
 	})
 }
 
+func TestSetForeignKeyInMultiOperationMigrations(t *testing.T) {
+	t.Parallel()
+
+	ExecuteTests(t, TestCases{
+		{
+			name: "rename referencing table, set fk",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_tables",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "items",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "varchar(255)",
+								},
+								{
+									Name:     "supplier_id",
+									Type:     "int",
+									Nullable: true,
+								},
+							},
+						},
+						&migrations.OpCreateTable{
+							Name: "suppliers",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "varchar(255)",
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_multi_operation",
+					Operations: migrations.Operations{
+						&migrations.OpRenameTable{
+							From: "items",
+							To:   "products",
+						},
+						&migrations.OpAlterColumn{
+							Table:  "products",
+							Column: "supplier_id",
+							References: &migrations.ForeignKeyReference{
+								Name:   "fk_products_suppliers",
+								Table:  "suppliers",
+								Column: "id",
+							},
+							Up:   "SELECT CASE WHEN EXISTS (SELECT 1 FROM suppliers WHERE suppliers.id = supplier_id) THEN supplier_id ELSE NULL END",
+							Down: "supplier_id",
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the suppliers table
+				MustInsert(t, db, schema, "02_multi_operation", "suppliers", map[string]string{
+					"id":   "1",
+					"name": "supplier1",
+				})
+
+				// Can insert a row into the products table that meets the FK constraint
+				MustInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":          "1",
+					"name":        "apples",
+					"supplier_id": "1",
+				})
+
+				// Can't insert a row into the products table that violates the FK constraint
+				MustNotInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":          "2",
+					"name":        "bananas",
+					"supplier_id": "999",
+				}, testutils.FKViolationErrorCode)
+
+				// Can insert a row into the old schema that violates the FK constraint
+				MustInsert(t, db, schema, "01_create_tables", "items", map[string]string{
+					"id":          "2",
+					"name":        "bananas",
+					"supplier_id": "999",
+				})
+
+				// The new view has the expected rows
+				rows := MustSelect(t, db, schema, "02_multi_operation", "products")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "supplier_id": 1},
+					{"id": 2, "name": "bananas", "supplier_id": nil},
+				}, rows)
+
+				// The old view has the expected rows
+				rows = MustSelect(t, db, schema, "01_create_tables", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "supplier_id": 1},
+					{"id": 2, "name": "bananas", "supplier_id": 999},
+				}, rows)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table has been cleaned up
+				TableMustBeCleanedUp(t, db, schema, "items", "name")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the products table that meets the FK constraint
+				MustInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":          "3",
+					"name":        "carrots",
+					"supplier_id": "1",
+				})
+
+				// Can't insert a row into the products table that violates the FK constraint
+				MustNotInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":          "4",
+					"name":        "durian",
+					"supplier_id": "999",
+				}, testutils.FKViolationErrorCode)
+
+				// The new view has the expected rows
+				rows := MustSelect(t, db, schema, "02_multi_operation", "products")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "supplier_id": 1},
+					{"id": 2, "name": "bananas", "supplier_id": nil},
+					{"id": 3, "name": "carrots", "supplier_id": 1},
+				}, rows)
+			},
+		},
+		{
+			name: "rename referencing table, rename referencing column, set fk",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_tables",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "items",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "varchar(255)",
+								},
+								{
+									Name:     "supplier_id",
+									Type:     "int",
+									Nullable: true,
+								},
+							},
+						},
+						&migrations.OpCreateTable{
+							Name: "suppliers",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "varchar(255)",
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_multi_operation",
+					Operations: migrations.Operations{
+						&migrations.OpRenameTable{
+							From: "items",
+							To:   "products",
+						},
+						&migrations.OpRenameColumn{
+							Table: "products",
+							From:  "supplier_id",
+							To:    "sup_id",
+						},
+						&migrations.OpAlterColumn{
+							Table:  "products",
+							Column: "sup_id",
+							References: &migrations.ForeignKeyReference{
+								Name:   "fk_products_suppliers",
+								Table:  "suppliers",
+								Column: "id",
+							},
+							Up:   "SELECT CASE WHEN EXISTS (SELECT 1 FROM suppliers WHERE suppliers.id = sup_id) THEN sup_id ELSE NULL END",
+							Down: "sup_id",
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the suppliers table
+				MustInsert(t, db, schema, "02_multi_operation", "suppliers", map[string]string{
+					"id":   "1",
+					"name": "supplier1",
+				})
+
+				// Can insert a row into the products table that meets the FK constraint
+				MustInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":     "1",
+					"name":   "apples",
+					"sup_id": "1",
+				})
+
+				// Can't insert a row into the products table that violates the FK constraint
+				MustNotInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":     "2",
+					"name":   "bananas",
+					"sup_id": "999",
+				}, testutils.FKViolationErrorCode)
+
+				// Can insert a row into the old schema that violates the FK constraint
+				MustInsert(t, db, schema, "01_create_tables", "items", map[string]string{
+					"id":          "2",
+					"name":        "bananas",
+					"supplier_id": "999",
+				})
+
+				// The new view has the expected rows
+				rows := MustSelect(t, db, schema, "02_multi_operation", "products")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "sup_id": 1},
+					{"id": 2, "name": "bananas", "sup_id": nil},
+				}, rows)
+
+				// The old view has the expected rows
+				rows = MustSelect(t, db, schema, "01_create_tables", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "supplier_id": 1},
+					{"id": 2, "name": "bananas", "supplier_id": 999},
+				}, rows)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table has been cleaned up
+				TableMustBeCleanedUp(t, db, schema, "items", "name")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the products table that meets the FK constraint
+				MustInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":     "3",
+					"name":   "carrots",
+					"sup_id": "1",
+				})
+
+				// Can't insert a row into the products table that violates the FK constraint
+				MustNotInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"id":     "4",
+					"name":   "durian",
+					"sup_id": "999",
+				}, testutils.FKViolationErrorCode)
+
+				// The new view has the expected rows
+				rows := MustSelect(t, db, schema, "02_multi_operation", "products")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "sup_id": 1},
+					{"id": 2, "name": "bananas", "sup_id": nil},
+					{"id": 3, "name": "carrots", "sup_id": 1},
+				}, rows)
+			},
+		},
+		{
+			name: "rename referenced table, set fk",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_tables",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "items",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "varchar(255)",
+								},
+								{
+									Name:     "supplier_id",
+									Type:     "int",
+									Nullable: true,
+								},
+							},
+						},
+						&migrations.OpCreateTable{
+							Name: "suppliers",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "varchar(255)",
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_multi_operation",
+					Operations: migrations.Operations{
+						&migrations.OpRenameTable{
+							From: "suppliers",
+							To:   "producers",
+						},
+						&migrations.OpAlterColumn{
+							Table:  "items",
+							Column: "supplier_id",
+							References: &migrations.ForeignKeyReference{
+								Name:   "fk_items_producers",
+								Table:  "producers",
+								Column: "id",
+							},
+							// Still have to refer to the renamed referenced table by its old name here which is not ideal
+							Up:   "SELECT CASE WHEN EXISTS (SELECT 1 FROM suppliers WHERE suppliers.id = supplier_id) THEN supplier_id ELSE NULL END",
+							Down: "supplier_id",
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the suppliers table
+				MustInsert(t, db, schema, "02_multi_operation", "producers", map[string]string{
+					"id":   "1",
+					"name": "producer1",
+				})
+
+				// Can insert a row into the products table that meets the FK constraint
+				MustInsert(t, db, schema, "02_multi_operation", "items", map[string]string{
+					"id":          "1",
+					"name":        "apples",
+					"supplier_id": "1",
+				})
+
+				// Can't insert a row into the products table that violates the FK constraint
+				MustNotInsert(t, db, schema, "02_multi_operation", "items", map[string]string{
+					"id":          "2",
+					"name":        "bananas",
+					"supplier_id": "999",
+				}, testutils.FKViolationErrorCode)
+
+				// Can insert a row into the old schema that violates the FK constraint
+				MustInsert(t, db, schema, "01_create_tables", "items", map[string]string{
+					"id":          "2",
+					"name":        "bananas",
+					"supplier_id": "999",
+				})
+
+				// The new view has the expected rows
+				rows := MustSelect(t, db, schema, "02_multi_operation", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "supplier_id": 1},
+					{"id": 2, "name": "bananas", "supplier_id": nil},
+				}, rows)
+
+				// The old view has the expected rows
+				rows = MustSelect(t, db, schema, "01_create_tables", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "supplier_id": 1},
+					{"id": 2, "name": "bananas", "supplier_id": 999},
+				}, rows)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table has been cleaned up
+				TableMustBeCleanedUp(t, db, schema, "items", "name")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the products table that meets the FK constraint
+				MustInsert(t, db, schema, "02_multi_operation", "items", map[string]string{
+					"id":          "3",
+					"name":        "carrots",
+					"supplier_id": "1",
+				})
+
+				// Can't insert a row into the products table that violates the FK constraint
+				MustNotInsert(t, db, schema, "02_multi_operation", "items", map[string]string{
+					"id":          "4",
+					"name":        "durian",
+					"supplier_id": "999",
+				}, testutils.FKViolationErrorCode)
+
+				// The new view has the expected rows
+				rows := MustSelect(t, db, schema, "02_multi_operation", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "supplier_id": 1},
+					{"id": 2, "name": "bananas", "supplier_id": nil},
+					{"id": 3, "name": "carrots", "supplier_id": 1},
+				}, rows)
+			},
+		},
+		{
+			name: "rename referenced column, set fk",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_tables",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "items",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "varchar(255)",
+								},
+								{
+									Name:     "supplier_id",
+									Type:     "int",
+									Nullable: true,
+								},
+							},
+						},
+						&migrations.OpCreateTable{
+							Name: "suppliers",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "int",
+									Pk:   true,
+								},
+								{
+									Name: "name",
+									Type: "varchar(255)",
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_multi_operation",
+					Operations: migrations.Operations{
+						&migrations.OpRenameColumn{
+							Table: "suppliers",
+							From:  "id",
+							To:    "supplier_id",
+						},
+						&migrations.OpAlterColumn{
+							Table:  "items",
+							Column: "supplier_id",
+							References: &migrations.ForeignKeyReference{
+								Name:   "fk_items_suppliers",
+								Table:  "suppliers",
+								Column: "supplier_id",
+							},
+							// Still have to refer to the renamed referenced column by its old name here which is not ideal
+							Up:   "SELECT CASE WHEN EXISTS (SELECT 1 FROM suppliers WHERE suppliers.id = supplier_id) THEN supplier_id ELSE NULL END",
+							Down: "supplier_id",
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the suppliers table
+				MustInsert(t, db, schema, "02_multi_operation", "suppliers", map[string]string{
+					"supplier_id": "1",
+					"name":        "producer1",
+				})
+
+				// Can insert a row into the products table that meets the FK constraint
+				MustInsert(t, db, schema, "02_multi_operation", "items", map[string]string{
+					"id":          "1",
+					"name":        "apples",
+					"supplier_id": "1",
+				})
+
+				// Can't insert a row into the products table that violates the FK constraint
+				MustNotInsert(t, db, schema, "02_multi_operation", "items", map[string]string{
+					"id":          "2",
+					"name":        "bananas",
+					"supplier_id": "999",
+				}, testutils.FKViolationErrorCode)
+
+				// Can insert a row into the old schema that violates the FK constraint
+				MustInsert(t, db, schema, "01_create_tables", "items", map[string]string{
+					"id":          "2",
+					"name":        "bananas",
+					"supplier_id": "999",
+				})
+
+				// The new view has the expected rows
+				rows := MustSelect(t, db, schema, "02_multi_operation", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "supplier_id": 1},
+					{"id": 2, "name": "bananas", "supplier_id": nil},
+				}, rows)
+
+				// The old view has the expected rows
+				rows = MustSelect(t, db, schema, "01_create_tables", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "supplier_id": 1},
+					{"id": 2, "name": "bananas", "supplier_id": 999},
+				}, rows)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table has been cleaned up
+				TableMustBeCleanedUp(t, db, schema, "items", "name")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the products table that meets the FK constraint
+				MustInsert(t, db, schema, "02_multi_operation", "items", map[string]string{
+					"id":          "3",
+					"name":        "carrots",
+					"supplier_id": "1",
+				})
+
+				// Can't insert a row into the products table that violates the FK constraint
+				MustNotInsert(t, db, schema, "02_multi_operation", "items", map[string]string{
+					"id":          "4",
+					"name":        "durian",
+					"supplier_id": "999",
+				}, testutils.FKViolationErrorCode)
+
+				// The new view has the expected rows
+				rows := MustSelect(t, db, schema, "02_multi_operation", "items")
+				assert.Equal(t, []map[string]any{
+					{"id": 1, "name": "apples", "supplier_id": 1},
+					{"id": 2, "name": "bananas", "supplier_id": nil},
+					{"id": 3, "name": "carrots", "supplier_id": 1},
+				}, rows)
+			},
+		},
+	})
+}
+
 func TestSetForeignKeyValidation(t *testing.T) {
 	t.Parallel()
 
@@ -1351,7 +1908,7 @@ func TestSetForeignKeyValidation(t *testing.T) {
 								Name:     "fk_users_doesntexist",
 								Table:    "users",
 								Column:   "id",
-								OnDelete: migrations.ForeignKeyReferenceOnDeleteNOACTION,
+								OnDelete: migrations.ForeignKeyActionNOACTION,
 							},
 							Up:   "SELECT CASE WHEN EXISTS (SELECT 1 FROM users WHERE users.id = user_id) THEN user_id ELSE NULL END",
 							Down: "user_id",
@@ -1375,7 +1932,7 @@ func TestSetForeignKeyValidation(t *testing.T) {
 								Name:     "fk_users_doesntexist",
 								Table:    "users",
 								Column:   "id",
-								OnDelete: migrations.ForeignKeyReferenceOnDeleteSETNULL,
+								OnDelete: migrations.ForeignKeyActionSETNULL,
 							},
 							Up:   "SELECT CASE WHEN EXISTS (SELECT 1 FROM users WHERE users.id = user_id) THEN user_id ELSE NULL END",
 							Down: "user_id",
