@@ -7,8 +7,9 @@ import (
 	"testing"
 
 	"github.com/stretchr/testify/assert"
+
+	"github.com/xataio/pgroll/internal/testutils"
 	"github.com/xataio/pgroll/pkg/migrations"
-	"github.com/xataio/pgroll/pkg/testutils"
 )
 
 func TestSetColumnUnique(t *testing.T) {
@@ -27,22 +28,22 @@ func TestSetColumnUnique(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "username",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 								{
 									Name:     "product",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 								{
 									Name:     "review",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 							},
 						},
@@ -80,32 +81,12 @@ func TestSetColumnUnique(t *testing.T) {
 				}, testutils.UniqueViolationErrorCode)
 			},
 			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
-				// The new (temporary) `review` column should not exist on the underlying table.
-				ColumnMustNotExist(t, db, schema, "reviews", migrations.TemporaryName("review"))
-
-				// The up function no longer exists.
-				FunctionMustNotExist(t, db, schema, migrations.TriggerFunctionName("reviews", "review"))
-				// The down function no longer exists.
-				FunctionMustNotExist(t, db, schema, migrations.TriggerFunctionName("reviews", migrations.TemporaryName("review")))
-
-				// The up trigger no longer exists.
-				TriggerMustNotExist(t, db, schema, "reviews", migrations.TriggerName("reviews", "review"))
-				// The down trigger no longer exists.
-				TriggerMustNotExist(t, db, schema, "reviews", migrations.TriggerName("reviews", migrations.TemporaryName("review")))
+				// The table is cleaned up; temporary columns, trigger functions and triggers no longer exist.
+				TableMustBeCleanedUp(t, db, schema, "reviews", "review")
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
-				// The new (temporary) `review` column should not exist on the underlying table.
-				ColumnMustNotExist(t, db, schema, "reviews", migrations.TemporaryName("review"))
-
-				// The up function no longer exists.
-				FunctionMustNotExist(t, db, schema, migrations.TriggerFunctionName("reviews", "review"))
-				// The down function no longer exists.
-				FunctionMustNotExist(t, db, schema, migrations.TriggerFunctionName("reviews", migrations.TemporaryName("review")))
-
-				// The up trigger no longer exists.
-				TriggerMustNotExist(t, db, schema, "reviews", migrations.TriggerName("reviews", "review"))
-				// The down trigger no longer exists.
-				TriggerMustNotExist(t, db, schema, "reviews", migrations.TriggerName("reviews", migrations.TemporaryName("review")))
+				// The table is cleaned up; temporary columns, trigger functions and triggers no longer exist.
+				TableMustBeCleanedUp(t, db, schema, "reviews", "review")
 
 				// Inserting values into the new schema that violate uniqueness should fail.
 				MustInsert(t, db, schema, "02_set_unique", "reviews", map[string]string{
@@ -128,22 +109,22 @@ func TestSetColumnUnique(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "username",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 								{
 									Name:     "product",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 								{
 									Name:     "review",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 							},
 						},
@@ -192,7 +173,7 @@ func TestSetColumnUnique(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:    "username",
@@ -272,12 +253,12 @@ func TestSetColumnUnique(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "name",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 							},
 						},
@@ -292,22 +273,22 @@ func TestSetColumnUnique(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "name",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 								{
 									Name:     "department_id",
 									Type:     "integer",
-									Nullable: ptr(true),
+									Nullable: true,
 									References: &migrations.ForeignKeyReference{
 										Name:     "fk_employee_department",
 										Table:    "departments",
 										Column:   "id",
-										OnDelete: migrations.ForeignKeyReferenceOnDeleteSETNULL,
+										OnDelete: migrations.ForeignKeyActionSETNULL,
 									},
 								},
 							},
@@ -331,13 +312,27 @@ func TestSetColumnUnique(t *testing.T) {
 			},
 			afterStart: func(t *testing.T, db *sql.DB, schema string) {
 				// A temporary FK constraint has been created on the temporary column
-				ValidatedForeignKeyMustExist(t, db, schema, "employees", migrations.DuplicationName("fk_employee_department"), withOnDeleteSetNull())
+				ValidatedForeignKeyMustExistWithReferentialAction(
+					t,
+					db,
+					schema,
+					"employees",
+					migrations.DuplicationName("fk_employee_department"),
+					migrations.ForeignKeyActionSETNULL,
+					migrations.ForeignKeyActionNOACTION)
 			},
 			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
 			},
 			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
 				// The foreign key constraint still exists on the column
-				ValidatedForeignKeyMustExist(t, db, schema, "employees", "fk_employee_department", withOnDeleteSetNull())
+				ValidatedForeignKeyMustExistWithReferentialAction(
+					t,
+					db,
+					schema,
+					"employees",
+					"fk_employee_department",
+					migrations.ForeignKeyActionSETNULL,
+					migrations.ForeignKeyActionNOACTION)
 			},
 		},
 		{
@@ -352,7 +347,7 @@ func TestSetColumnUnique(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name: "username",
@@ -414,12 +409,12 @@ func TestSetColumnUnique(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "username",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 								},
 								{
 									Name: "product",
@@ -475,12 +470,12 @@ func TestSetColumnUnique(t *testing.T) {
 								{
 									Name: "id",
 									Type: "serial",
-									Pk:   ptr(true),
+									Pk:   true,
 								},
 								{
 									Name:     "username",
 									Type:     "text",
-									Nullable: ptr(false),
+									Nullable: false,
 									Comment:  ptr("the name of the user"),
 								},
 								{
@@ -526,5 +521,281 @@ func TestSetColumnUnique(t *testing.T) {
 		//
 		// In that case it should be possible to test that existing unique constraints are
 		// preserved when adding a new unique constraint covering the same column.
+		{
+			name: "validate that the constraint name is not already taken",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_add_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "reviews",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "serial",
+									Pk:   true,
+								},
+								{
+									Name:     "username",
+									Type:     "text",
+									Nullable: false,
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_set_unique",
+					Operations: migrations.Operations{
+						&migrations.OpAlterColumn{
+							Table:  "reviews",
+							Column: "username",
+							Unique: &migrations.UniqueConstraint{
+								Name: "reviews_username_key",
+							},
+						},
+					},
+				},
+				{
+					Name: "03_set_unique_again",
+					Operations: migrations.Operations{
+						&migrations.OpAlterColumn{
+							Table:  "reviews",
+							Column: "username",
+							Unique: &migrations.UniqueConstraint{
+								Name: "reviews_username_key",
+							},
+						},
+					},
+				},
+			},
+			wantStartErr: migrations.ConstraintAlreadyExistsError{
+				Table:      "reviews",
+				Constraint: "reviews_username_key",
+			},
+			afterStart:    func(t *testing.T, db *sql.DB, schema string) {},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {},
+		},
+	})
+}
+
+func TestSetUniqueInMultiOperationMigrations(t *testing.T) {
+	t.Parallel()
+
+	ExecuteTests(t, TestCases{
+		{
+			name: "rename table, set unique",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "items",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "serial",
+									Pk:   true,
+								},
+								{
+									Name:     "name",
+									Type:     "varchar(255)",
+									Nullable: true,
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_multi_operation",
+					Operations: migrations.Operations{
+						&migrations.OpRenameTable{
+							From: "items",
+							To:   "products",
+						},
+						&migrations.OpAlterColumn{
+							Table:  "products",
+							Column: "name",
+							Unique: &migrations.UniqueConstraint{
+								Name: "products_name_unique",
+							},
+							Up:   "name || '-' || floor(random()*100000)::text",
+							Down: "name",
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row that meets the constraint into the new view
+				MustInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"name": "apple",
+				})
+
+				// Can't insert a row that violates the constraint into the new view
+				MustNotInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"name": "apple",
+				}, testutils.UniqueViolationErrorCode)
+
+				// Can insert a row that violates the constraint into the old view
+				MustInsert(t, db, schema, "01_create_table", "items", map[string]string{
+					"name": "apple",
+				})
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table has been cleaned up
+				TableMustBeCleanedUp(t, db, schema, "items", "name")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row that meets the constraint into the new view
+				MustInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"name": "banana",
+				})
+
+				// Can't insert a row that violates the constraint into the new view
+				MustNotInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"name": "banana",
+				}, testutils.UniqueViolationErrorCode)
+			},
+		},
+		{
+			name: "rename table, rename column, set unique",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_create_table",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "items",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "serial",
+									Pk:   true,
+								},
+								{
+									Name:     "name",
+									Type:     "varchar(255)",
+									Nullable: true,
+								},
+							},
+						},
+					},
+				},
+				{
+					Name: "02_multi_operation",
+					Operations: migrations.Operations{
+						&migrations.OpRenameTable{
+							From: "items",
+							To:   "products",
+						},
+						&migrations.OpRenameColumn{
+							Table: "products",
+							From:  "name",
+							To:    "item_name",
+						},
+						&migrations.OpAlterColumn{
+							Table:  "products",
+							Column: "item_name",
+							Unique: &migrations.UniqueConstraint{
+								Name: "products_name_unique",
+							},
+							Up:   "item_name || '-' || floor(random()*100000)::text",
+							Down: "item_name",
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row that meets the constraint into the new view
+				MustInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"item_name": "apple",
+				})
+
+				// Can't insert a row that violates the constraint into the new view
+				MustNotInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"item_name": "apple",
+				}, testutils.UniqueViolationErrorCode)
+
+				// Can insert a row that violates the constraint into the old view
+				MustInsert(t, db, schema, "01_create_table", "items", map[string]string{
+					"name": "apple",
+				})
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table has been cleaned up
+				TableMustBeCleanedUp(t, db, schema, "items", "name")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row that meets the constraint into the new view
+				MustInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"item_name": "banana",
+				})
+
+				// Can't insert a row that violates the constraint into the new view
+				MustNotInsert(t, db, schema, "02_multi_operation", "products", map[string]string{
+					"item_name": "banana",
+				}, testutils.UniqueViolationErrorCode)
+			},
+		},
+		{
+			name: "create table, set unique",
+			migrations: []migrations.Migration{
+				{
+					Name: "01_multi_operation",
+					Operations: migrations.Operations{
+						&migrations.OpCreateTable{
+							Name: "items",
+							Columns: []migrations.Column{
+								{
+									Name: "id",
+									Type: "serial",
+									Pk:   true,
+								},
+								{
+									Name:     "name",
+									Type:     "varchar(255)",
+									Nullable: true,
+								},
+							},
+						},
+						&migrations.OpAlterColumn{
+							Table:  "items",
+							Column: "name",
+							Unique: &migrations.UniqueConstraint{
+								Name: "items_name_unique",
+							},
+							Up:   "name || '-' || floor(random()*100000)::text",
+							Down: "name",
+						},
+					},
+				},
+			},
+			afterStart: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the new (only) schema that meets the constraint
+				MustInsert(t, db, schema, "01_multi_operation", "items", map[string]string{
+					"name": "apple",
+				})
+
+				// Can't insert a row into the new (only) schema that violates the constraint
+				MustNotInsert(t, db, schema, "01_multi_operation", "items", map[string]string{
+					"name": "apple",
+				}, testutils.UniqueViolationErrorCode)
+			},
+			afterRollback: func(t *testing.T, db *sql.DB, schema string) {
+				// The table has been dropped
+				TableMustNotExist(t, db, schema, "items")
+			},
+			afterComplete: func(t *testing.T, db *sql.DB, schema string) {
+				// Can insert a row into the new (only) schema that meets the constraint
+				MustInsert(t, db, schema, "01_multi_operation", "items", map[string]string{
+					"name": "apple",
+				})
+
+				// Can't insert a row into the new (only) schema that violates the constraint
+				MustNotInsert(t, db, schema, "01_multi_operation", "items", map[string]string{
+					"name": "apple",
+				}, testutils.UniqueViolationErrorCode)
+			},
+		},
 	})
 }
