@@ -19,12 +19,9 @@ const Function = `CREATE OR REPLACE FUNCTION {{ .Name | qi }}()
         INTO search_path
         FROM current_setting('search_path');
 
-      IF search_path {{- if eq .Direction "up" }} != {{- else }} = {{- end }} {{ .LatestSchema | ql }} {{ if .TestExpr  -}} AND {{ .TestExpr }} {{ end -}} THEN
+      IF search_path {{- if eq .Direction "up" }} != {{- else }} = {{- end }} {{ .LatestSchema | ql }} THEN
         NEW.{{ .PhysicalColumn | qi  }} = {{ .SQL }};
-      {{- if .ElseExpr }}
-      ELSE
-        {{ .ElseExpr }};
-      {{- end }}
+        NEW.{{ .NeedsBackfillColumn | qi }} = false;
       END IF;
 
       RETURN NEW;
