@@ -5,6 +5,8 @@ package migrations
 import (
 	"bytes"
 	"context"
+	"database/sql"
+	"fmt"
 	"text/template"
 
 	"github.com/lib/pq"
@@ -21,52 +23,55 @@ const (
 	TriggerDirectionDown TriggerDirection = "down"
 )
 
+const CNeedsBackfillColumn = "_pgroll_needs_backfill"
+
 type triggerConfig struct {
-	Name           string
-	Direction      TriggerDirection
-	Columns        map[string]schema.Column
-	SchemaName     string
-	TableName      string
-	PhysicalColumn string
-	LatestSchema   string
-	TestExpr       string
-	ElseExpr       string
-	SQL            string
+	Name                string
+	Direction           TriggerDirection
+	Columns             map[string]*schema.Column
+	SchemaName          string
+	TableName           string
+	PhysicalColumn      string
+	LatestSchema        string
+	SQL                 string
+	NeedsBackfillColumn string
 }
 
-func createTrigger(ctx context.Context, conn db.DB, tr SQLTransformer, cfg triggerConfig) error {
-	sql, err := tr.TransformSQL(cfg.SQL)
-	if err != nil {
-		return err
+func createTrigger(ctx context.Context, conn db.DB, cfg triggerConfig) error {
+	// Parenthesize the up/down SQL if it's not parenthesized already
+	if len(cfg.SQL) > 0 && cfg.SQL[0] != '(' {
+		cfg.SQL = "(" + cfg.SQL + ")"
 	}
 
-	if len(sql) > 0 && sql[0] != '(' {
-		sql = "(" + sql + ")"
-	}
-
-	cfg.SQL = sql
+	cfg.NeedsBackfillColumn = CNeedsBackfillColumn
 
 	funcSQL, err := buildFunction(cfg)
 	if err != nil {
 		return err
 	}
 
-	_, err = conn.ExecContext(ctx, funcSQL)
-	if err != nil {
-		return err
-	}
-
 	triggerSQL, err := buildTrigger(cfg)
 	if err != nil {
 		return err
 	}
 
-	_, err = conn.ExecContext(ctx, triggerSQL)
-	if err != nil {
-		return err
-	}
+	return conn.WithRetryableTransaction(ctx, func(ctx context.Context, tx *sql.Tx) error {
+		_, err := conn.ExecContext(ctx,
+			fmt.Sprintf("ALTER TABLE %s ADD COLUMN IF NOT EXISTS %s boolean DEFAULT true",
+				pq.QuoteIdentifier(cfg.TableName),
+				pq.QuoteIdentifier(CNeedsBackfillColumn)))
+		if err != nil {
+			return err
+		}
 
-	return nil
+		_, err = conn.ExecContext(ctx, funcSQL)
+		if err != nil {
+			return err
+		}
+
+		_, err = conn.ExecContext(ctx, triggerSQL)
+		return err
+	})
 }
 
 func buildFunction(cfg triggerConfig) (string, error) {
