@@ -20,17 +20,18 @@ func TestBuildFunction(t *testing.T) {
 			config: triggerConfig{
 				Name:      "triggerName",
 				Direction: TriggerDirectionUp,
-				Columns: map[string]schema.Column{
+				Columns: map[string]*schema.Column{
 					"id":       {Name: "id", Type: "int"},
 					"username": {Name: "username", Type: "text"},
 					"product":  {Name: "product", Type: "text"},
 					"review":   {Name: "review", Type: "text"},
 				},
-				SchemaName:     "public",
-				LatestSchema:   "public_01_migration_name",
-				TableName:      "reviews",
-				PhysicalColumn: "_pgroll_new_review",
-				SQL:            "product || 'is good'",
+				SchemaName:          "public",
+				LatestSchema:        "public_01_migration_name",
+				TableName:           "reviews",
+				PhysicalColumn:      "_pgroll_new_review",
+				NeedsBackfillColumn: CNeedsBackfillColumn,
+				SQL:                 "product || 'is good'",
 			},
 			expected: `CREATE OR REPLACE FUNCTION "triggerName"()
     RETURNS TRIGGER
@@ -50,51 +51,7 @@ func TestBuildFunction(t *testing.T) {
 
       IF search_path != 'public_01_migration_name' THEN
         NEW."_pgroll_new_review" = product || 'is good';
-      END IF;
-
-      RETURN NEW;
-    END; $$
-`,
-		},
-		{
-			name: "complete up trigger",
-			config: triggerConfig{
-				Name:      "triggerName",
-				Direction: TriggerDirectionUp,
-				Columns: map[string]schema.Column{
-					"id":       {Name: "id", Type: "int"},
-					"username": {Name: "username", Type: "text"},
-					"product":  {Name: "product", Type: "text"},
-					"review":   {Name: "review", Type: "text"},
-				},
-				SchemaName:     "public",
-				LatestSchema:   "public_01_migration_name",
-				TableName:      "reviews",
-				TestExpr:       `NEW."review" IS NULL`,
-				PhysicalColumn: "_pgroll_new_review",
-				ElseExpr:       `NEW."_pgroll_new_review" = NEW."review"`,
-				SQL:            "product || 'is good'",
-			},
-			expected: `CREATE OR REPLACE FUNCTION "triggerName"()
-    RETURNS TRIGGER
-    LANGUAGE PLPGSQL
-    AS $$
-    DECLARE
-      "id" "public"."reviews"."id"%TYPE := NEW."id";
-      "product" "public"."reviews"."product"%TYPE := NEW."product";
-      "review" "public"."reviews"."review"%TYPE := NEW."review";
-      "username" "public"."reviews"."username"%TYPE := NEW."username";
-      latest_schema text;
-      search_path text;
-    BEGIN
-      SELECT current_setting
-        INTO search_path
-        FROM current_setting('search_path');
-
-      IF search_path != 'public_01_migration_name' AND NEW."review" IS NULL THEN
-        NEW."_pgroll_new_review" = product || 'is good';
-      ELSE
-        NEW."_pgroll_new_review" = NEW."review";
+        NEW."_pgroll_needs_backfill" = false;
       END IF;
 
       RETURN NEW;
@@ -106,17 +63,18 @@ func TestBuildFunction(t *testing.T) {
 			config: triggerConfig{
 				Name:      "triggerName",
 				Direction: TriggerDirectionDown,
-				Columns: map[string]schema.Column{
+				Columns: map[string]*schema.Column{
 					"id":       {Name: "id", Type: "int"},
 					"username": {Name: "username", Type: "text"},
 					"product":  {Name: "product", Type: "text"},
 					"review":   {Name: "review", Type: "text"},
 				},
-				SchemaName:     "public",
-				LatestSchema:   "public_01_migration_name",
-				TableName:      "reviews",
-				PhysicalColumn: "review",
-				SQL:            `NEW."_pgroll_new_review"`,
+				SchemaName:          "public",
+				LatestSchema:        "public_01_migration_name",
+				TableName:           "reviews",
+				PhysicalColumn:      "review",
+				NeedsBackfillColumn: CNeedsBackfillColumn,
+				SQL:                 `NEW."_pgroll_new_review"`,
 			},
 			expected: `CREATE OR REPLACE FUNCTION "triggerName"()
     RETURNS TRIGGER
@@ -136,6 +94,7 @@ func TestBuildFunction(t *testing.T) {
 
       IF search_path = 'public_01_migration_name' THEN
         NEW."review" = NEW."_pgroll_new_review";
+        NEW."_pgroll_needs_backfill" = false;
       END IF;
 
       RETURN NEW;
@@ -147,18 +106,19 @@ func TestBuildFunction(t *testing.T) {
 			config: triggerConfig{
 				Name:      "triggerName",
 				Direction: TriggerDirectionDown,
-				Columns: map[string]schema.Column{
+				Columns: map[string]*schema.Column{
 					"id":       {Name: "id", Type: "int"},
 					"username": {Name: "username", Type: "text"},
 					"product":  {Name: "product", Type: "text"},
 					"review":   {Name: "review", Type: "text"},
 					"rating":   {Name: "_pgroll_new_rating", Type: "integer"},
 				},
-				SchemaName:     "public",
-				LatestSchema:   "public_01_migration_name",
-				TableName:      "reviews",
-				PhysicalColumn: "rating",
-				SQL:            `CAST(rating as text)`,
+				SchemaName:          "public",
+				LatestSchema:        "public_01_migration_name",
+				TableName:           "reviews",
+				PhysicalColumn:      "rating",
+				NeedsBackfillColumn: CNeedsBackfillColumn,
+				SQL:                 `CAST(rating as text)`,
 			},
 			expected: `CREATE OR REPLACE FUNCTION "triggerName"()
     RETURNS TRIGGER
@@ -179,6 +139,7 @@ func TestBuildFunction(t *testing.T) {
 
       IF search_path = 'public_01_migration_name' THEN
         NEW."rating" = CAST(rating as text);
+        NEW."_pgroll_needs_backfill" = false;
       END IF;
 
       RETURN NEW;
