@@ -7,21 +7,17 @@ import (
 	"database/sql"
 	"fmt"
 	"strings"
-	"time"
 
 	"github.com/lib/pq"
 
 	"github.com/xataio/pgroll/pkg/db"
-	"github.com/xataio/pgroll/pkg/migrations"
 	"github.com/xataio/pgroll/pkg/state"
 )
 
 type PGVersion int
 
 const (
-	PGVersion15              PGVersion     = 15
-	DefaultBackfillBatchSize int           = 1000
-	DefaultBackfillDelay     time.Duration = 0
+	PGVersion15 PGVersion = 15
 )
 
 var ErrMismatchedMigration = fmt.Errorf("remote migration does not match local migration")
@@ -41,11 +37,7 @@ type Roll struct {
 	migrationHooks MigrationHooks
 	state          *state.State
 	pgVersion      PGVersion
-	sqlTransformer migrations.SQLTransformer
-
-	backfillBatchSize  int
-	backfillBatchDelay time.Duration
-	skipValidation     bool
+	skipValidation bool
 }
 
 // New creates a new Roll instance
@@ -54,9 +46,6 @@ func New(ctx context.Context, pgURL, schema string, state *state.State, opts ...
 	for _, o := range opts {
 		o(rollOpts)
 	}
-	if rollOpts.backfillBatchSize <= 0 {
-		rollOpts.backfillBatchSize = DefaultBackfillBatchSize
-	}
 
 	conn, err := setupConn(ctx, pgURL, schema, *rollOpts)
 	if err != nil {
@@ -69,24 +58,14 @@ func New(ctx context.Context, pgURL, schema string, state *state.State, opts ...
 		return nil, fmt.Errorf("unable to retrieve postgres version: %w", err)
 	}
 
-	var sqlTransformer migrations.SQLTransformer = migrations.SQLTransformerFunc(
-		func(sql string) (string, error) { return sql, nil },
-	)
-	if rollOpts.sqlTransformer != nil {
-		sqlTransformer = rollOpts.sqlTransformer
-	}
-
 	return &Roll{
 		pgConn:                   &db.RDB{DB: conn},
 		schema:                   schema,
 		state:                    state,
 		pgVersion:                pgMajorVersion,
-		sqlTransformer:           sqlTransformer,
 		disableVersionSchemas:    rollOpts.disableVersionSchemas,
 		noVersionSchemaForRawSQL: rollOpts.noVersionSchemaForRawSQL,
 		migrationHooks:           rollOpts.migrationHooks,
-		backfillBatchSize:        rollOpts.backfillBatchSize,
-		backfillBatchDelay:       rollOpts.backfillBatchDelay,
 		skipValidation:           rollOpts.skipValidation,
 	}, nil
 }
