@@ -76,7 +76,7 @@ STABLE;
 -- Get the name of the previous version of the schema, or NULL if there is none.
 -- This ignores previous versions for which no version schema exists, such as
 -- versions corresponding to inferred migrations.
-CREATE OR REPLACE FUNCTION placeholder.previous_version (schemaname name)
+CREATE OR REPLACE FUNCTION placeholder.previous_version (schemaname name, includeInferred boolean)
     RETURNS text
     AS $$
     WITH RECURSIVE ancestors AS (
@@ -107,10 +107,17 @@ CREATE OR REPLACE FUNCTION placeholder.previous_version (schemaname name)
             a.name
         FROM
             ancestors a
-        JOIN information_schema.schemata s ON s.schema_name = schemaname || '_' || a.name
     WHERE
-        migration_type = 'pgroll'
-        AND a.depth > 0
+        a.depth > 0
+        AND (includeInferred
+            OR (a.migration_type = 'pgroll'
+                AND EXISTS (
+                    SELECT
+                        s.schema_name
+                    FROM
+                        information_schema.schemata s
+                    WHERE
+                        s.schema_name = schemaname || '_' || a.name)))
     ORDER BY
         a.depth ASC
     LIMIT 1;
@@ -163,7 +170,21 @@ BEGIN
                                         array_agg(e.enumlabel ORDER BY e.enumsortorder)
                                     FROM pg_enum AS e
                                 WHERE
-                                    e.enumtypid = tp.oid) AS enumValues FROM pg_attribute AS attr
+                                    e.enumtypid = tp.oid) AS enumValues, CASE WHEN tp.typtype = 'b' THEN
+                                    'base'
+                                WHEN tp.typtype = 'c' THEN
+                                    'composite'
+                                WHEN tp.typtype = 'd' THEN
+                                    'domain'
+                                WHEN tp.typtype = 'e' THEN
+                                    'enum'
+                                WHEN tp.typtype = 'p' THEN
+                                    'pseudo'
+                                WHEN tp.typtype = 'r' THEN
+                                    'range'
+                                WHEN tp.typtype = 'm' THEN
+                                    'multirange'
+                                END AS postgresType FROM pg_attribute AS attr
                                 INNER JOIN pg_type AS tp ON attr.atttypid = tp.oid
                                 LEFT JOIN pg_attrdef AS def ON attr.attrelid = def.adrelid
                                     AND attr.attnum = def.adnum
@@ -183,10 +204,10 @@ BEGIN
                                 AND pg_attribute.attnum = ANY (pg_index.indkey)
                                 AND indisprimary), 'indexes', (
                                 SELECT
-                                    COALESCE(json_object_agg(ix_details.name, json_build_object('name', ix_details.name, 'unique', ix_details.indisunique, 'columns', ix_details.columns, 'predicate', ix_details.predicate, 'method', ix_details.method, 'definition', ix_details.definition)), '{}'::json)
+                                    COALESCE(json_object_agg(ix_details.name, json_build_object('name', ix_details.name, 'unique', ix_details.indisunique, 'exclusion', ix_details.indisexclusion, 'columns', ix_details.columns, 'predicate', ix_details.predicate, 'method', ix_details.method, 'definition', ix_details.definition)), '{}'::json)
                             FROM (
                                 SELECT
-                                    replace(reverse(split_part(reverse(pi.indexrelid::regclass::text), '.', 1)), '"', '') AS name, pi.indisunique, array_agg(a.attname) AS columns, pg_get_expr(pi.indpred, t.oid) AS predicate, am.amname AS method, pg_get_indexdef(pi.indexrelid) AS definition
+                                    replace(reverse(split_part(reverse(pi.indexrelid::regclass::text), '.', 1)), '"', '') AS name, pi.indisunique, pi.indisexclusion, array_agg(a.attname) AS columns, pg_get_expr(pi.indpred, t.oid) AS predicate, am.amname AS method, pg_get_indexdef(pi.indexrelid) AS definition
                                 FROM pg_index pi
                                 JOIN pg_attribute a ON a.attrelid = pi.indrelid
                                     AND a.attnum = ANY (pi.indkey)
@@ -195,10 +216,10 @@ BEGIN
                                 WHERE
                                     indrelid = t.oid::regclass GROUP BY pi.indexrelid, pi.indisunique, pi.indpred, am.amname) AS ix_details), 'checkConstraints', (
                         SELECT
-                            COALESCE(json_object_agg(cc_details.conname, json_build_object('name', cc_details.conname, 'columns', cc_details.columns, 'definition', cc_details.definition)), '{}'::json)
+                            COALESCE(json_object_agg(cc_details.conname, json_build_object('name', cc_details.conname, 'columns', cc_details.columns, 'definition', cc_details.definition, 'noInherit', cc_details.connoinherit)), '{}'::json)
                         FROM (
                             SELECT
-                                cc_constraint.conname, array_agg(cc_attr.attname ORDER BY cc_constraint.conkey::int[]) AS columns, pg_get_constraintdef(cc_constraint.oid) AS definition FROM pg_constraint AS cc_constraint
+                                cc_constraint.conname, array_agg(cc_attr.attname ORDER BY cc_constraint.conkey::int[]) AS columns, pg_get_constraintdef(cc_constraint.oid) AS definition, cc_constraint.connoinherit FROM pg_constraint AS cc_constraint
                             INNER JOIN pg_attribute cc_attr ON cc_attr.attrelid = cc_constraint.conrelid
                                 AND cc_attr.attnum = ANY (cc_constraint.conkey)
                             WHERE
@@ -213,40 +234,69 @@ BEGIN
                                     AND uc_attr.attnum = ANY (uc_constraint.conkey)
                                 WHERE
                                     uc_constraint.conrelid = t.oid
-                                    AND uc_constraint.contype = 'u' GROUP BY uc_constraint.oid, uc_constraint.conname) AS uc_details), 'foreignKeys', (
+                                    AND uc_constraint.contype = 'u' GROUP BY uc_constraint.oid, uc_constraint.conname) AS uc_details), 'excludeConstraints', (
                                 SELECT
-                                    COALESCE(json_object_agg(fk_details.conname, json_build_object('name', fk_details.conname, 'columns', fk_details.columns, 'referencedTable', fk_details.referencedTable, 'referencedColumns', fk_details.referencedColumns, 'onDelete', fk_details.onDelete)), '{}'::json)
+                                    COALESCE(json_object_agg(xc_details.conname, json_build_object('name', xc_details.conname, 'columns', xc_details.columns, 'definition', xc_details.definition, 'predicate', xc_details.predicate, 'method', xc_details.method)), '{}'::json)
                                 FROM (
                                     SELECT
-                                        fk_info.conname AS conname, fk_info.columns AS columns, fk_info.relname AS referencedTable, array_agg(ref_attr.attname ORDER BY ref_attr.attname) AS referencedColumns, CASE WHEN fk_info.confdeltype = 'a' THEN
-                                        'NO ACTION'
-                                    WHEN fk_info.confdeltype = 'r' THEN
-                                        'RESTRICT'
-                                    WHEN fk_info.confdeltype = 'c' THEN
-                                        'CASCADE'
-                                    WHEN fk_info.confdeltype = 'd' THEN
-                                        'SET DEFAULT'
-                                    WHEN fk_info.confdeltype = 'n' THEN
-                                        'SET NULL'
-                                    END AS onDelete FROM (
+                                        xc_constraint.conname, array_agg(xc_attr.attname ORDER BY xc_constraint.conkey::int[]) AS columns, pg_get_expr(pi.indpred, t.oid) AS predicate, am.amname AS method, pg_get_constraintdef(xc_constraint.oid) AS definition FROM pg_constraint AS xc_constraint
+                                    INNER JOIN pg_attribute xc_attr ON xc_attr.attrelid = xc_constraint.conrelid
+                                        AND xc_attr.attnum = ANY (xc_constraint.conkey)
+                                    JOIN pg_index pi ON pi.indexrelid = xc_constraint.conindid
+                                    JOIN pg_class cls ON cls.oid = pi.indexrelid
+                                    JOIN pg_am am ON am.oid = cls.relam
+                                    WHERE
+                                        xc_constraint.conrelid = t.oid
+                                        AND xc_constraint.contype = 'x' GROUP BY xc_constraint.oid, xc_constraint.conname, pi.indpred, pi.indexrelid, am.amname) AS xc_details), 'foreignKeys', (
+                                    SELECT
+                                        COALESCE(json_object_agg(fk_details.conname, json_build_object('name', fk_details.conname, 'columns', fk_details.columns, 'referencedTable', fk_details.referencedTable, 'referencedColumns', fk_details.referencedColumns, 'matchType', fk_details.matchType, 'onDelete', fk_details.onDelete, 'onUpdate', fk_details.onUpdate)), '{}'::json)
+                                    FROM (
                                         SELECT
-                                            fk_constraint.conname, fk_constraint.conrelid, fk_constraint.confrelid, fk_constraint.confkey, fk_cl.relname, fk_constraint.confdeltype, array_agg(fk_attr.attname ORDER BY fk_attr.attname) AS columns FROM pg_constraint AS fk_constraint
-                                        INNER JOIN pg_class fk_cl ON fk_constraint.confrelid = fk_cl.oid -- join the referenced table
-                                        INNER JOIN pg_attribute fk_attr ON fk_attr.attrelid = fk_constraint.conrelid
-                                            AND fk_attr.attnum = ANY (fk_constraint.conkey) -- join the columns of the referencing table
-                                        WHERE
-                                            fk_constraint.conrelid = t.oid
-                                            AND fk_constraint.contype = 'f' GROUP BY fk_constraint.conrelid, fk_constraint.conname, fk_constraint.confrelid, fk_cl.relname, fk_constraint.confkey, fk_constraint.confdeltype) AS fk_info
-                                        INNER JOIN pg_attribute ref_attr ON ref_attr.attrelid = fk_info.confrelid
-                                            AND ref_attr.attnum = ANY (fk_info.confkey) -- join the columns of the referenced table
-                                    GROUP BY fk_info.conname, fk_info.conrelid, fk_info.columns, fk_info.confrelid, fk_info.confdeltype, fk_info.relname) AS fk_details))), '{}'::json)
-                    FROM pg_class AS t
-                    INNER JOIN pg_namespace AS ns ON t.relnamespace = ns.oid
-                    LEFT JOIN pg_description AS descr ON t.oid = descr.objoid
-                        AND descr.objsubid = 0
-                    WHERE
-                        ns.nspname = schemaname
-                        AND t.relkind IN ('r', 'p') -- tables only (ignores views, materialized views & foreign tables)
+                                            fk_info.conname AS conname, fk_info.columns AS columns, fk_info.relname AS referencedTable, array_agg(ref_attr.attname ORDER BY ref_attr.attname) AS referencedColumns, CASE WHEN fk_info.confmatchtype = 'f' THEN
+                                            'FULL'
+                                        WHEN fk_info.confmatchtype = 'p' THEN
+                                            'PARTIAL'
+                                        WHEN fk_info.confmatchtype = 's' THEN
+                                            'SIMPLE'
+                                        END AS matchType, CASE WHEN fk_info.confdeltype = 'a' THEN
+                                            'NO ACTION'
+                                        WHEN fk_info.confdeltype = 'r' THEN
+                                            'RESTRICT'
+                                        WHEN fk_info.confdeltype = 'c' THEN
+                                            'CASCADE'
+                                        WHEN fk_info.confdeltype = 'd' THEN
+                                            'SET DEFAULT'
+                                        WHEN fk_info.confdeltype = 'n' THEN
+                                            'SET NULL'
+                                        END AS onDelete, CASE WHEN fk_info.confupdtype = 'a' THEN
+                                            'NO ACTION'
+                                        WHEN fk_info.confupdtype = 'r' THEN
+                                            'RESTRICT'
+                                        WHEN fk_info.confupdtype = 'c' THEN
+                                            'CASCADE'
+                                        WHEN fk_info.confupdtype = 'd' THEN
+                                            'SET DEFAULT'
+                                        WHEN fk_info.confupdtype = 'n' THEN
+                                            'SET NULL'
+                                        END AS onUpdate FROM (
+                                            SELECT
+                                                fk_constraint.conname, fk_constraint.conrelid, fk_constraint.confrelid, fk_constraint.confkey, fk_cl.relname, fk_constraint.confmatchtype, fk_constraint.confdeltype, fk_constraint.confupdtype, array_agg(fk_attr.attname ORDER BY fk_attr.attname) AS columns FROM pg_constraint AS fk_constraint
+                                            INNER JOIN pg_class fk_cl ON fk_constraint.confrelid = fk_cl.oid -- join the referenced table
+                                            INNER JOIN pg_attribute fk_attr ON fk_attr.attrelid = fk_constraint.conrelid
+                                                AND fk_attr.attnum = ANY (fk_constraint.conkey) -- join the columns of the referencing table
+                                            WHERE
+                                                fk_constraint.conrelid = t.oid
+                                                AND fk_constraint.contype = 'f' GROUP BY fk_constraint.conrelid, fk_constraint.conname, fk_constraint.confrelid, fk_cl.relname, fk_constraint.confkey, fk_constraint.confmatchtype, fk_constraint.confdeltype, fk_constraint.confupdtype) AS fk_info
+                                            INNER JOIN pg_attribute ref_attr ON ref_attr.attrelid = fk_info.confrelid
+                                                AND ref_attr.attnum = ANY (fk_info.confkey) -- join the columns of the referenced table
+                                        GROUP BY fk_info.conname, fk_info.conrelid, fk_info.columns, fk_info.confrelid, fk_info.confmatchtype, fk_info.confdeltype, fk_info.confupdtype, fk_info.relname) AS fk_details))), '{}'::json)
+                        FROM pg_class AS t
+                        INNER JOIN pg_namespace AS ns ON t.relnamespace = ns.oid
+                        LEFT JOIN pg_description AS descr ON t.oid = descr.objoid
+                            AND descr.objsubid = 0
+                        WHERE
+                            ns.nspname = schemaname
+                            AND t.relkind IN ('r', 'p') -- tables only (ignores views, materialized views & foreign tables)
 )) INTO tables;
     RETURN tables;
 END;
