@@ -136,11 +136,11 @@ func (s *State) LatestVersion(ctx context.Context, schema string) (*string, erro
 }
 
 // PreviousVersion returns the name of the previous version schema
-func (s *State) PreviousVersion(ctx context.Context, schema string) (*string, error) {
+func (s *State) PreviousVersion(ctx context.Context, schema string, includeInferred bool) (*string, error) {
 	var parent *string
 	err := s.pgConn.QueryRowContext(ctx,
-		fmt.Sprintf("SELECT %s.previous_version($1)", pq.QuoteIdentifier(s.schema)),
-		schema).Scan(&parent)
+		fmt.Sprintf("SELECT %s.previous_version($1, $2)", pq.QuoteIdentifier(s.schema)),
+		schema, includeInferred).Scan(&parent)
 	if err != nil {
 		return nil, err
 	}
@@ -250,6 +250,26 @@ func (s *State) ReadSchema(ctx context.Context, schemaName string) (*schema.Sche
 	return &sc, nil
 }
 
+// SchemaAfterMigration reads the schema after the migration `version` was
+// applied to `schemaName`
+func (s *State) SchemaAfterMigration(ctx context.Context, schemaName, version string) (*schema.Schema, error) {
+	sql := fmt.Sprintf("SELECT resulting_schema FROM %s.migrations WHERE schema=$1 AND name=$2", pq.QuoteIdentifier(s.schema))
+
+	var rawSchema []byte
+	err := s.pgConn.QueryRowContext(ctx, sql, schemaName, version).Scan(&rawSchema)
+	if err != nil {
+		return nil, err
+	}
+
+	var sc schema.Schema
+	err = json.Unmarshal(rawSchema, &sc)
+	if err != nil {
+		return nil, fmt.Errorf("unable to unmarshal schema: %w", err)
+	}
+
+	return &sc, nil
+}
+
 // Rollback removes a migration from the state (we consider it rolled back, as if it never started)
 func (s *State) Rollback(ctx context.Context, schema, name string) error {
 	res, err := s.pgConn.ExecContext(ctx, fmt.Sprintf("DELETE FROM %s.migrations WHERE schema=$1 AND name=$2 AND done=$3", pq.QuoteIdentifier(s.schema)), schema, name, false)
