@@ -377,7 +377,7 @@ func TestReadSchema(t *testing.T) {
 				createStmt: "",
 				wantSchema: &schema.Schema{
 					Name:   "public",
-					Tables: map[string]schema.Table{},
+					Tables: map[string]*schema.Table{},
 				},
 			},
 			{
@@ -385,15 +385,16 @@ func TestReadSchema(t *testing.T) {
 				createStmt: "CREATE TABLE public.table1 ()",
 				wantSchema: &schema.Schema{
 					Name: "public",
-					Tables: map[string]schema.Table{
+					Tables: map[string]*schema.Table{
 						"table1": {
-							Name:              "table1",
-							Columns:           map[string]schema.Column{},
-							PrimaryKey:        []string{},
-							Indexes:           map[string]schema.Index{},
-							CheckConstraints:  map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{},
-							ForeignKeys:       map[string]schema.ForeignKey{},
+							Name:               "table1",
+							Columns:            map[string]*schema.Column{},
+							PrimaryKey:         []string{},
+							Indexes:            map[string]*schema.Index{},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+							ForeignKeys:        map[string]*schema.ForeignKey{},
 						},
 					},
 				},
@@ -403,21 +404,23 @@ func TestReadSchema(t *testing.T) {
 				createStmt: "CREATE TABLE public.table1 (id int)",
 				wantSchema: &schema.Schema{
 					Name: "public",
-					Tables: map[string]schema.Table{
+					Tables: map[string]*schema.Table{
 						"table1": {
 							Name: "table1",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"id": {
-									Name:     "id",
-									Type:     "integer",
-									Nullable: true,
+									Name:         "id",
+									Type:         "integer",
+									Nullable:     true,
+									PostgresType: "base",
 								},
 							},
-							PrimaryKey:        []string{},
-							Indexes:           map[string]schema.Index{},
-							CheckConstraints:  map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{},
-							ForeignKeys:       map[string]schema.ForeignKey{},
+							PrimaryKey:         []string{},
+							Indexes:            map[string]*schema.Index{},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+							ForeignKeys:        map[string]*schema.ForeignKey{},
 						},
 					},
 				},
@@ -427,19 +430,20 @@ func TestReadSchema(t *testing.T) {
 				createStmt: "CREATE TABLE public.table1 (id int NOT NULL, CONSTRAINT id_unique UNIQUE(id))",
 				wantSchema: &schema.Schema{
 					Name: "public",
-					Tables: map[string]schema.Table{
+					Tables: map[string]*schema.Table{
 						"table1": {
 							Name: "table1",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"id": {
-									Name:     "id",
-									Type:     "integer",
-									Nullable: false,
-									Unique:   true,
+									Name:         "id",
+									Type:         "integer",
+									Nullable:     false,
+									Unique:       true,
+									PostgresType: "base",
 								},
 							},
 							PrimaryKey: []string{},
-							Indexes: map[string]schema.Index{
+							Indexes: map[string]*schema.Index{
 								"id_unique": {
 									Name:       "id_unique",
 									Unique:     true,
@@ -448,14 +452,15 @@ func TestReadSchema(t *testing.T) {
 									Definition: "CREATE UNIQUE INDEX id_unique ON public.table1 USING btree (id)",
 								},
 							},
-							CheckConstraints: map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{
+							CheckConstraints: map[string]*schema.CheckConstraint{},
+							UniqueConstraints: map[string]*schema.UniqueConstraint{
 								"id_unique": {
 									Name:    "id_unique",
 									Columns: []string{"id"},
 								},
 							},
-							ForeignKeys: map[string]schema.ForeignKey{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+							ForeignKeys:        map[string]*schema.ForeignKey{},
 						},
 					},
 				},
@@ -465,23 +470,25 @@ func TestReadSchema(t *testing.T) {
 				createStmt: "CREATE TABLE public.table1 (id int, name text); CREATE INDEX idx_name ON public.table1 (name)",
 				wantSchema: &schema.Schema{
 					Name: "public",
-					Tables: map[string]schema.Table{
+					Tables: map[string]*schema.Table{
 						"table1": {
 							Name: "table1",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"id": {
-									Name:     "id",
-									Type:     "integer",
-									Nullable: true,
+									Name:         "id",
+									Type:         "integer",
+									Nullable:     true,
+									PostgresType: "base",
 								},
 								"name": {
-									Name:     "name",
-									Type:     "text",
-									Nullable: true,
+									Name:         "name",
+									Type:         "text",
+									Nullable:     true,
+									PostgresType: "base",
 								},
 							},
 							PrimaryKey: []string{},
-							Indexes: map[string]schema.Index{
+							Indexes: map[string]*schema.Index{
 								"idx_name": {
 									Name:       "idx_name",
 									Unique:     false,
@@ -490,9 +497,10 @@ func TestReadSchema(t *testing.T) {
 									Definition: "CREATE INDEX idx_name ON public.table1 USING btree (name)",
 								},
 							},
-							CheckConstraints:  map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{},
-							ForeignKeys:       map[string]schema.ForeignKey{},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+							ForeignKeys:        map[string]*schema.ForeignKey{},
 						},
 					},
 				},
@@ -502,19 +510,20 @@ func TestReadSchema(t *testing.T) {
 				createStmt: "CREATE TABLE public.table1 (id int PRIMARY KEY); CREATE TABLE public.table2 (fk int NOT NULL, CONSTRAINT fk_fkey FOREIGN KEY (fk) REFERENCES public.table1 (id))",
 				wantSchema: &schema.Schema{
 					Name: "public",
-					Tables: map[string]schema.Table{
+					Tables: map[string]*schema.Table{
 						"table1": {
 							Name: "table1",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"id": {
-									Name:     "id",
-									Type:     "integer",
-									Nullable: false,
-									Unique:   true,
+									Name:         "id",
+									Type:         "integer",
+									Nullable:     false,
+									Unique:       true,
+									PostgresType: "base",
 								},
 							},
 							PrimaryKey: []string{"id"},
-							Indexes: map[string]schema.Index{
+							Indexes: map[string]*schema.Index{
 								"table1_pkey": {
 									Name:       "table1_pkey",
 									Unique:     true,
@@ -523,32 +532,37 @@ func TestReadSchema(t *testing.T) {
 									Definition: "CREATE UNIQUE INDEX table1_pkey ON public.table1 USING btree (id)",
 								},
 							},
-							CheckConstraints:  map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{},
-							ForeignKeys:       map[string]schema.ForeignKey{},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+							ForeignKeys:        map[string]*schema.ForeignKey{},
 						},
 						"table2": {
 							Name: "table2",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"fk": {
-									Name:     "fk",
-									Type:     "integer",
-									Nullable: false,
+									Name:         "fk",
+									Type:         "integer",
+									Nullable:     false,
+									PostgresType: "base",
 								},
 							},
 							PrimaryKey: []string{},
-							Indexes:    map[string]schema.Index{},
-							ForeignKeys: map[string]schema.ForeignKey{
+							Indexes:    map[string]*schema.Index{},
+							ForeignKeys: map[string]*schema.ForeignKey{
 								"fk_fkey": {
 									Name:              "fk_fkey",
 									Columns:           []string{"fk"},
 									ReferencedTable:   "table1",
 									ReferencedColumns: []string{"id"},
+									MatchType:         "SIMPLE",
 									OnDelete:          "NO ACTION",
+									OnUpdate:          "NO ACTION",
 								},
 							},
-							CheckConstraints:  map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
 						},
 					},
 				},
@@ -558,19 +572,20 @@ func TestReadSchema(t *testing.T) {
 				createStmt: "CREATE TABLE public.table1 (id int PRIMARY KEY); CREATE TABLE public.table2 (fk int NOT NULL, CONSTRAINT fk_fkey FOREIGN KEY (fk) REFERENCES public.table1 (id) ON DELETE CASCADE)",
 				wantSchema: &schema.Schema{
 					Name: "public",
-					Tables: map[string]schema.Table{
+					Tables: map[string]*schema.Table{
 						"table1": {
 							Name: "table1",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"id": {
-									Name:     "id",
-									Type:     "integer",
-									Nullable: false,
-									Unique:   true,
+									Name:         "id",
+									Type:         "integer",
+									Nullable:     false,
+									Unique:       true,
+									PostgresType: "base",
 								},
 							},
 							PrimaryKey: []string{"id"},
-							Indexes: map[string]schema.Index{
+							Indexes: map[string]*schema.Index{
 								"table1_pkey": {
 									Name:       "table1_pkey",
 									Unique:     true,
@@ -579,32 +594,161 @@ func TestReadSchema(t *testing.T) {
 									Definition: "CREATE UNIQUE INDEX table1_pkey ON public.table1 USING btree (id)",
 								},
 							},
-							CheckConstraints:  map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{},
-							ForeignKeys:       map[string]schema.ForeignKey{},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+							ForeignKeys:        map[string]*schema.ForeignKey{},
 						},
 						"table2": {
 							Name: "table2",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"fk": {
-									Name:     "fk",
-									Type:     "integer",
-									Nullable: false,
+									Name:         "fk",
+									Type:         "integer",
+									Nullable:     false,
+									PostgresType: "base",
 								},
 							},
 							PrimaryKey: []string{},
-							Indexes:    map[string]schema.Index{},
-							ForeignKeys: map[string]schema.ForeignKey{
+							Indexes:    map[string]*schema.Index{},
+							ForeignKeys: map[string]*schema.ForeignKey{
 								"fk_fkey": {
 									Name:              "fk_fkey",
 									Columns:           []string{"fk"},
 									ReferencedTable:   "table1",
 									ReferencedColumns: []string{"id"},
+									MatchType:         "SIMPLE",
 									OnDelete:          "CASCADE",
+									OnUpdate:          "NO ACTION",
 								},
 							},
-							CheckConstraints:  map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+						},
+					},
+				},
+			},
+			{
+				name:       "foreign key with ON DELETE CASCADE ON UPDATE CASCADE",
+				createStmt: "CREATE TABLE public.table1 (id int PRIMARY KEY); CREATE TABLE public.table2 (fk int NOT NULL, CONSTRAINT fk_fkey FOREIGN KEY (fk) REFERENCES public.table1 (id) ON DELETE CASCADE ON UPDATE CASCADE)",
+				wantSchema: &schema.Schema{
+					Name: "public",
+					Tables: map[string]*schema.Table{
+						"table1": {
+							Name: "table1",
+							Columns: map[string]*schema.Column{
+								"id": {
+									Name:         "id",
+									Type:         "integer",
+									Nullable:     false,
+									Unique:       true,
+									PostgresType: "base",
+								},
+							},
+							PrimaryKey: []string{"id"},
+							Indexes: map[string]*schema.Index{
+								"table1_pkey": {
+									Name:       "table1_pkey",
+									Unique:     true,
+									Columns:    []string{"id"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE UNIQUE INDEX table1_pkey ON public.table1 USING btree (id)",
+								},
+							},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+							ForeignKeys:        map[string]*schema.ForeignKey{},
+						},
+						"table2": {
+							Name: "table2",
+							Columns: map[string]*schema.Column{
+								"fk": {
+									Name:         "fk",
+									Type:         "integer",
+									Nullable:     false,
+									PostgresType: "base",
+								},
+							},
+							PrimaryKey: []string{},
+							Indexes:    map[string]*schema.Index{},
+							ForeignKeys: map[string]*schema.ForeignKey{
+								"fk_fkey": {
+									Name:              "fk_fkey",
+									Columns:           []string{"fk"},
+									ReferencedTable:   "table1",
+									ReferencedColumns: []string{"id"},
+									MatchType:         "SIMPLE",
+									OnDelete:          "CASCADE",
+									OnUpdate:          "CASCADE",
+								},
+							},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+						},
+					},
+				},
+			},
+			{
+				name:       "foreign key with MATCH full ON DELETE CASCADE",
+				createStmt: "CREATE TABLE public.table1 (id int PRIMARY KEY); CREATE TABLE public.table2 (fk int NOT NULL, CONSTRAINT fk_fkey FOREIGN KEY (fk) REFERENCES public.table1 (id) MATCH FULL ON DELETE CASCADE)",
+				wantSchema: &schema.Schema{
+					Name: "public",
+					Tables: map[string]*schema.Table{
+						"table1": {
+							Name: "table1",
+							Columns: map[string]*schema.Column{
+								"id": {
+									Name:         "id",
+									Type:         "integer",
+									Nullable:     false,
+									Unique:       true,
+									PostgresType: "base",
+								},
+							},
+							PrimaryKey: []string{"id"},
+							Indexes: map[string]*schema.Index{
+								"table1_pkey": {
+									Name:       "table1_pkey",
+									Unique:     true,
+									Columns:    []string{"id"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE UNIQUE INDEX table1_pkey ON public.table1 USING btree (id)",
+								},
+							},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+							ForeignKeys:        map[string]*schema.ForeignKey{},
+						},
+						"table2": {
+							Name: "table2",
+							Columns: map[string]*schema.Column{
+								"fk": {
+									Name:         "fk",
+									Type:         "integer",
+									Nullable:     false,
+									PostgresType: "base",
+								},
+							},
+							PrimaryKey: []string{},
+							Indexes:    map[string]*schema.Index{},
+							ForeignKeys: map[string]*schema.ForeignKey{
+								"fk_fkey": {
+									Name:              "fk_fkey",
+									Columns:           []string{"fk"},
+									ReferencedTable:   "table1",
+									ReferencedColumns: []string{"id"},
+									MatchType:         "FULL",
+									OnDelete:          "CASCADE",
+									OnUpdate:          "NO ACTION",
+								},
+							},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
 						},
 					},
 				},
@@ -614,24 +758,26 @@ func TestReadSchema(t *testing.T) {
 				createStmt: "CREATE TABLE public.table1 (id int PRIMARY KEY, age INTEGER, CONSTRAINT age_check CHECK (age > 18));",
 				wantSchema: &schema.Schema{
 					Name: "public",
-					Tables: map[string]schema.Table{
+					Tables: map[string]*schema.Table{
 						"table1": {
 							Name: "table1",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"id": {
-									Name:     "id",
-									Type:     "integer",
-									Nullable: false,
-									Unique:   true,
+									Name:         "id",
+									Type:         "integer",
+									Nullable:     false,
+									Unique:       true,
+									PostgresType: "base",
 								},
 								"age": {
-									Name:     "age",
-									Type:     "integer",
-									Nullable: true,
+									Name:         "age",
+									Type:         "integer",
+									Nullable:     true,
+									PostgresType: "base",
 								},
 							},
 							PrimaryKey: []string{"id"},
-							Indexes: map[string]schema.Index{
+							Indexes: map[string]*schema.Index{
 								"table1_pkey": {
 									Name:       "table1_pkey",
 									Unique:     true,
@@ -640,15 +786,50 @@ func TestReadSchema(t *testing.T) {
 									Definition: "CREATE UNIQUE INDEX table1_pkey ON public.table1 USING btree (id)",
 								},
 							},
-							ForeignKeys: map[string]schema.ForeignKey{},
-							CheckConstraints: map[string]schema.CheckConstraint{
+							ForeignKeys: map[string]*schema.ForeignKey{},
+							CheckConstraints: map[string]*schema.CheckConstraint{
 								"age_check": {
 									Name:       "age_check",
 									Columns:    []string{"age"},
 									Definition: "CHECK ((age > 18))",
+									NoInherit:  false,
 								},
 							},
-							UniqueConstraints: map[string]schema.UniqueConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+						},
+					},
+				},
+			},
+			{
+				name:       "check constraint with no inherit",
+				createStmt: "CREATE TABLE public.table1 (age INTEGER, CONSTRAINT age_check CHECK (age > 18) NO INHERIT);",
+				wantSchema: &schema.Schema{
+					Name: "public",
+					Tables: map[string]*schema.Table{
+						"table1": {
+							Name: "table1",
+							Columns: map[string]*schema.Column{
+								"age": {
+									Name:         "age",
+									Type:         "integer",
+									Nullable:     true,
+									PostgresType: "base",
+								},
+							},
+							PrimaryKey:  []string{},
+							Indexes:     map[string]*schema.Index{},
+							ForeignKeys: map[string]*schema.ForeignKey{},
+							CheckConstraints: map[string]*schema.CheckConstraint{
+								"age_check": {
+									Name:       "age_check",
+									Columns:    []string{"age"},
+									Definition: "CHECK ((age > 18)) NO INHERIT",
+									NoInherit:  true,
+								},
+							},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
 						},
 					},
 				},
@@ -658,25 +839,27 @@ func TestReadSchema(t *testing.T) {
 				createStmt: "CREATE TABLE public.table1 (id int PRIMARY KEY, name TEXT, CONSTRAINT name_unique UNIQUE(name) );",
 				wantSchema: &schema.Schema{
 					Name: "public",
-					Tables: map[string]schema.Table{
+					Tables: map[string]*schema.Table{
 						"table1": {
 							Name: "table1",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"id": {
-									Name:     "id",
-									Type:     "integer",
-									Nullable: false,
-									Unique:   true,
+									Name:         "id",
+									Type:         "integer",
+									Nullable:     false,
+									Unique:       true,
+									PostgresType: "base",
 								},
 								"name": {
-									Name:     "name",
-									Type:     "text",
-									Unique:   true,
-									Nullable: true,
+									Name:         "name",
+									Type:         "text",
+									Unique:       true,
+									Nullable:     true,
+									PostgresType: "base",
 								},
 							},
 							PrimaryKey: []string{"id"},
-							Indexes: map[string]schema.Index{
+							Indexes: map[string]*schema.Index{
 								"table1_pkey": {
 									Name:       "table1_pkey",
 									Unique:     true,
@@ -692,14 +875,15 @@ func TestReadSchema(t *testing.T) {
 									Definition: "CREATE UNIQUE INDEX name_unique ON public.table1 USING btree (name)",
 								},
 							},
-							ForeignKeys:      map[string]schema.ForeignKey{},
-							CheckConstraints: map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{
+							ForeignKeys:      map[string]*schema.ForeignKey{},
+							CheckConstraints: map[string]*schema.CheckConstraint{},
+							UniqueConstraints: map[string]*schema.UniqueConstraint{
 								"name_unique": {
 									Name:    "name_unique",
 									Columns: []string{"name"},
 								},
 							},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
 						},
 					},
 				},
@@ -709,25 +893,27 @@ func TestReadSchema(t *testing.T) {
 				createStmt: "CREATE TABLE public.table1 (id int PRIMARY KEY, name TEXT, CONSTRAINT name_id_unique UNIQUE(id, name));",
 				wantSchema: &schema.Schema{
 					Name: "public",
-					Tables: map[string]schema.Table{
+					Tables: map[string]*schema.Table{
 						"table1": {
 							Name: "table1",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"id": {
-									Name:     "id",
-									Type:     "integer",
-									Nullable: false,
-									Unique:   true,
+									Name:         "id",
+									Type:         "integer",
+									Nullable:     false,
+									Unique:       true,
+									PostgresType: "base",
 								},
 								"name": {
-									Name:     "name",
-									Type:     "text",
-									Nullable: true,
-									Unique:   false,
+									Name:         "name",
+									Type:         "text",
+									Nullable:     true,
+									Unique:       false,
+									PostgresType: "base",
 								},
 							},
 							PrimaryKey: []string{"id"},
-							Indexes: map[string]schema.Index{
+							Indexes: map[string]*schema.Index{
 								"table1_pkey": {
 									Name:       "table1_pkey",
 									Unique:     true,
@@ -743,14 +929,57 @@ func TestReadSchema(t *testing.T) {
 									Definition: "CREATE UNIQUE INDEX name_id_unique ON public.table1 USING btree (id, name)",
 								},
 							},
-							ForeignKeys:      map[string]schema.ForeignKey{},
-							CheckConstraints: map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{
+							ForeignKeys:      map[string]*schema.ForeignKey{},
+							CheckConstraints: map[string]*schema.CheckConstraint{},
+							UniqueConstraints: map[string]*schema.UniqueConstraint{
 								"name_id_unique": {
 									Name:    "name_id_unique",
 									Columns: []string{"id", "name"},
 								},
 							},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+						},
+					},
+				},
+			},
+			{
+				name:       "exclusion constraint",
+				createStmt: "CREATE TABLE public.table1 (name TEXT, CONSTRAINT name_unique EXCLUDE USING btree (name WITH =));",
+				wantSchema: &schema.Schema{
+					Name: "public",
+					Tables: map[string]*schema.Table{
+						"table1": {
+							Name: "table1",
+							Columns: map[string]*schema.Column{
+								"name": {
+									Name:         "name",
+									Type:         "text",
+									Nullable:     true,
+									PostgresType: "base",
+								},
+							},
+							PrimaryKey: []string{},
+							Indexes: map[string]*schema.Index{
+								"name_unique": {
+									Name:       "name_unique",
+									Exclusion:  true,
+									Unique:     false,
+									Columns:    []string{"name"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE INDEX name_unique ON public.table1 USING btree (name)",
+								},
+							},
+							ForeignKeys:       map[string]*schema.ForeignKey{},
+							CheckConstraints:  map[string]*schema.CheckConstraint{},
+							UniqueConstraints: map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{
+								"name_unique": {
+									Name:       "name_unique",
+									Columns:    []string{"name"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "EXCLUDE USING btree (name WITH =)",
+								},
+							},
 						},
 					},
 				},
@@ -768,23 +997,25 @@ func TestReadSchema(t *testing.T) {
             CONSTRAINT fk_customer_product FOREIGN KEY (customer_id, product_id) REFERENCES products (customer_id, product_id));`,
 				wantSchema: &schema.Schema{
 					Name: "public",
-					Tables: map[string]schema.Table{
+					Tables: map[string]*schema.Table{
 						"products": {
 							Name: "products",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"customer_id": {
-									Name:     "customer_id",
-									Type:     "integer",
-									Nullable: false,
+									Name:         "customer_id",
+									Type:         "integer",
+									Nullable:     false,
+									PostgresType: "base",
 								},
 								"product_id": {
-									Name:     "product_id",
-									Type:     "integer",
-									Nullable: false,
+									Name:         "product_id",
+									Type:         "integer",
+									Nullable:     false,
+									PostgresType: "base",
 								},
 							},
 							PrimaryKey: []string{"customer_id", "product_id"},
-							Indexes: map[string]schema.Index{
+							Indexes: map[string]*schema.Index{
 								"products_pkey": {
 									Name:       "products_pkey",
 									Unique:     true,
@@ -793,37 +1024,124 @@ func TestReadSchema(t *testing.T) {
 									Definition: "CREATE UNIQUE INDEX products_pkey ON public.products USING btree (customer_id, product_id)",
 								},
 							},
-							ForeignKeys:       map[string]schema.ForeignKey{},
-							CheckConstraints:  map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{},
+							ForeignKeys:        map[string]*schema.ForeignKey{},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
 						},
 						"orders": {
 							Name: "orders",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"customer_id": {
-									Name:     "customer_id",
-									Type:     "integer",
-									Nullable: false,
+									Name:         "customer_id",
+									Type:         "integer",
+									Nullable:     false,
+									PostgresType: "base",
 								},
 								"product_id": {
-									Name:     "product_id",
-									Type:     "integer",
-									Nullable: false,
+									Name:         "product_id",
+									Type:         "integer",
+									Nullable:     false,
+									PostgresType: "base",
 								},
 							},
 							PrimaryKey: []string{},
-							Indexes:    map[string]schema.Index{},
-							ForeignKeys: map[string]schema.ForeignKey{
+							Indexes:    map[string]*schema.Index{},
+							ForeignKeys: map[string]*schema.ForeignKey{
 								"fk_customer_product": {
 									Name:              "fk_customer_product",
 									Columns:           []string{"customer_id", "product_id"},
 									ReferencedTable:   "products",
 									ReferencedColumns: []string{"customer_id", "product_id"},
+									MatchType:         "SIMPLE",
 									OnDelete:          "NO ACTION",
+									OnUpdate:          "NO ACTION",
 								},
 							},
-							CheckConstraints:  map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+						},
+					},
+				},
+			},
+			{
+				name: "multicolumn foreign key constraint with on update action",
+				createStmt: `CREATE TABLE products(
+          customer_id INT NOT NULL, 
+          product_id INT NOT NULL, 
+          PRIMARY KEY(customer_id, product_id));
+
+          CREATE TABLE orders(
+            customer_id INT NOT NULL, 
+            product_id INT NOT NULL, 
+            CONSTRAINT fk_customer_product FOREIGN KEY (customer_id, product_id) REFERENCES products (customer_id, product_id) ON UPDATE CASCADE);`,
+				wantSchema: &schema.Schema{
+					Name: "public",
+					Tables: map[string]*schema.Table{
+						"products": {
+							Name: "products",
+							Columns: map[string]*schema.Column{
+								"customer_id": {
+									Name:         "customer_id",
+									Type:         "integer",
+									Nullable:     false,
+									PostgresType: "base",
+								},
+								"product_id": {
+									Name:         "product_id",
+									Type:         "integer",
+									Nullable:     false,
+									PostgresType: "base",
+								},
+							},
+							PrimaryKey: []string{"customer_id", "product_id"},
+							Indexes: map[string]*schema.Index{
+								"products_pkey": {
+									Name:       "products_pkey",
+									Unique:     true,
+									Columns:    []string{"customer_id", "product_id"},
+									Method:     string(migrations.OpCreateIndexMethodBtree),
+									Definition: "CREATE UNIQUE INDEX products_pkey ON public.products USING btree (customer_id, product_id)",
+								},
+							},
+							ForeignKeys:        map[string]*schema.ForeignKey{},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+						},
+						"orders": {
+							Name: "orders",
+							Columns: map[string]*schema.Column{
+								"customer_id": {
+									Name:         "customer_id",
+									Type:         "integer",
+									Nullable:     false,
+									PostgresType: "base",
+								},
+								"product_id": {
+									Name:         "product_id",
+									Type:         "integer",
+									Nullable:     false,
+									PostgresType: "base",
+								},
+							},
+							PrimaryKey: []string{},
+							Indexes:    map[string]*schema.Index{},
+							ForeignKeys: map[string]*schema.ForeignKey{
+								"fk_customer_product": {
+									Name:              "fk_customer_product",
+									Columns:           []string{"customer_id", "product_id"},
+									ReferencedTable:   "products",
+									ReferencedColumns: []string{"customer_id", "product_id"},
+									MatchType:         "SIMPLE",
+									OnDelete:          "NO ACTION",
+									OnUpdate:          "CASCADE",
+								},
+							},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
 						},
 					},
 				},
@@ -833,23 +1151,25 @@ func TestReadSchema(t *testing.T) {
 				createStmt: "CREATE TABLE public.table1 (a text, b text); CREATE INDEX idx_ab ON public.table1 (a, b);",
 				wantSchema: &schema.Schema{
 					Name: "public",
-					Tables: map[string]schema.Table{
+					Tables: map[string]*schema.Table{
 						"table1": {
 							Name: "table1",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"a": {
-									Name:     "a",
-									Type:     "text",
-									Nullable: true,
+									Name:         "a",
+									Type:         "text",
+									Nullable:     true,
+									PostgresType: "base",
 								},
 								"b": {
-									Name:     "b",
-									Type:     "text",
-									Nullable: true,
+									Name:         "b",
+									Type:         "text",
+									Nullable:     true,
+									PostgresType: "base",
 								},
 							},
 							PrimaryKey: []string{},
-							Indexes: map[string]schema.Index{
+							Indexes: map[string]*schema.Index{
 								"idx_ab": {
 									Name:       "idx_ab",
 									Unique:     false,
@@ -858,9 +1178,10 @@ func TestReadSchema(t *testing.T) {
 									Definition: "CREATE INDEX idx_ab ON public.table1 USING btree (a, b)",
 								},
 							},
-							ForeignKeys:       map[string]schema.ForeignKey{},
-							CheckConstraints:  map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{},
+							ForeignKeys:        map[string]*schema.ForeignKey{},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
 						},
 					},
 				},
@@ -870,21 +1191,23 @@ func TestReadSchema(t *testing.T) {
 				createStmt: "CREATE DOMAIN email_type AS varchar(255); CREATE TABLE public.table1 (a email_type);",
 				wantSchema: &schema.Schema{
 					Name: "public",
-					Tables: map[string]schema.Table{
+					Tables: map[string]*schema.Table{
 						"table1": {
 							Name: "table1",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"a": {
-									Name:     "a",
-									Type:     "public.email_type",
-									Nullable: true,
+									Name:         "a",
+									Type:         "public.email_type",
+									Nullable:     true,
+									PostgresType: "domain",
 								},
 							},
-							PrimaryKey:        []string{},
-							Indexes:           map[string]schema.Index{},
-							ForeignKeys:       map[string]schema.ForeignKey{},
-							CheckConstraints:  map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{},
+							PrimaryKey:         []string{},
+							Indexes:            map[string]*schema.Index{},
+							ForeignKeys:        map[string]*schema.ForeignKey{},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
 						},
 					},
 				},
@@ -894,27 +1217,90 @@ func TestReadSchema(t *testing.T) {
 				createStmt: "CREATE TYPE review AS ENUM ('good', 'bad', 'ugly'); CREATE TABLE public.table1 (name text, review review);",
 				wantSchema: &schema.Schema{
 					Name: "public",
-					Tables: map[string]schema.Table{
+					Tables: map[string]*schema.Table{
 						"table1": {
 							Name: "table1",
-							Columns: map[string]schema.Column{
+							Columns: map[string]*schema.Column{
 								"name": {
-									Name:     "name",
-									Type:     "text",
-									Nullable: true,
+									Name:         "name",
+									Type:         "text",
+									Nullable:     true,
+									PostgresType: "base",
 								},
 								"review": {
-									Name:       "review",
-									Type:       "public.review",
-									Nullable:   true,
-									EnumValues: []string{"good", "bad", "ugly"},
+									Name:         "review",
+									Type:         "public.review",
+									Nullable:     true,
+									EnumValues:   []string{"good", "bad", "ugly"},
+									PostgresType: "enum",
 								},
 							},
-							PrimaryKey:        []string{},
-							Indexes:           map[string]schema.Index{},
-							ForeignKeys:       map[string]schema.ForeignKey{},
-							CheckConstraints:  map[string]schema.CheckConstraint{},
-							UniqueConstraints: map[string]schema.UniqueConstraint{},
+							PrimaryKey:         []string{},
+							Indexes:            map[string]*schema.Index{},
+							ForeignKeys:        map[string]*schema.ForeignKey{},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
+						},
+					},
+				},
+			},
+			{
+				name: "postgres type types",
+				createStmt: `
+					CREATE TYPE comptype AS (f1 int, f2 text);
+					CREATE TYPE review AS ENUM ('good', 'bad', 'ugly');
+					CREATE TYPE float8_range AS RANGE (subtype = float8, subtype_diff = float8mi);
+					CREATE DOMAIN us_postal_code AS TEXT
+						CHECK(
+							VALUE ~ '^\d{5}$'
+							OR VALUE ~ '^\d{5}-\d{4}$'
+						);
+					CREATE TABLE public.table1 (id bigint, comp_col comptype, enum_col review, range_col float8_range, domain_col us_postal_code);`,
+				wantSchema: &schema.Schema{
+					Name: "public",
+					Tables: map[string]*schema.Table{
+						"table1": {
+							Name: "table1",
+							Columns: map[string]*schema.Column{
+								"id": {
+									Name:         "id",
+									Type:         "bigint",
+									Nullable:     true,
+									PostgresType: "base",
+								},
+								"comp_col": {
+									Name:         "comp_col",
+									Type:         "public.comptype",
+									Nullable:     true,
+									PostgresType: "composite",
+								},
+								"enum_col": {
+									Name:         "enum_col",
+									Type:         "public.review",
+									Nullable:     true,
+									PostgresType: "enum",
+									EnumValues:   []string{"good", "bad", "ugly"},
+								},
+								"range_col": {
+									Name:         "range_col",
+									Type:         "public.float8_range",
+									Nullable:     true,
+									PostgresType: "range",
+								},
+								"domain_col": {
+									Name:         "domain_col",
+									Type:         "public.us_postal_code",
+									Nullable:     true,
+									PostgresType: "domain",
+								},
+							},
+							PrimaryKey:         []string{},
+							Indexes:            map[string]*schema.Index{},
+							ForeignKeys:        map[string]*schema.ForeignKey{},
+							CheckConstraints:   map[string]*schema.CheckConstraint{},
+							UniqueConstraints:  map[string]*schema.UniqueConstraint{},
+							ExcludeConstraints: map[string]*schema.ExcludeConstraint{},
 						},
 					},
 				},
